-------------------------------------------------------------------------------
-- ESO Master Recipe List
-------------------------------------------------------------------------------
--[[
-- Copyright (c) 2016-2018 James A. Keene (Phinix) All rights reserved.
--
-- Permission is hereby granted, free of charge, to any person obtaining
-- a copy of this software and associated documentation (the "Software"),
-- to operate the Software for personal use only. Permission is NOT granted
-- to modify, merge, publish, distribute, sublicense, re-upload, and/or sell
-- copies of the Software. Additionally, licensed use of the Software
-- will be subject to the following:
--
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-- HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-- OTHER DEALINGS IN THE SOFTWARE.
--
-------------------------------------------------------------------------------
--
-- DISCLAIMER:
--
-- This Add-on is not created by, affiliated with or sponsored by ZeniMax
-- Media Inc. or its affiliates. The Elder Scrolls® and related logos are
-- registered trademarks or trademarks of ZeniMax Media Inc. in the United
-- States and/or other countries. All rights reserved.
--
-- You can read the full terms at:
-- https://account.elderscrollsonline.com/add-on-terms

Helpful language functions:
/script SetCVar("Language.2", "en")
/script SetCVar("Language.2", "fr")
/script SetCVar("Language.2", "de")
--]]

local ESOMRL = _G['ESOMRL']
local L = ESOMRL.Strings
local DB = ESOMRL.DB
ESOMRL.Name = 'MasterRecipeList'
ESOMRL.Author = '|c66ccffPhinix|r'
ESOMRL.Version = '1.55.2'

local pRecipes = 564	-- Number of Provisioning recipes currently in the game.
local fRecipes = 1638	-- Number of Furniture recipes currently in the game.
local cOpts, aOpts

local setPage = 1
local setMode = 1
local editText = 0
local recountTracked = 0
local checkingShown = 0
local playerLoaded = 0
local stationItem = 0
local stnorecs = 0
local textInput = ""
local stationSelection = ""
local searchTable = {}
local qualityCap = {}
local nameList = {}
local dataNames = {}
local statNodes = {}
local WritIng = {}
local openTab
local currentChar
local charnameDropdown
local StationControl
local RecipeItemTooltipControl
local IngredientItemTooltipControl
local FCOItemSaverActive
local PREVIEW = LibStub("LibPreview")

--------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Get and set saved variables.
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function TierIndex(index, tier)
	if tier == 2 then
		index = index + 42
	elseif tier == 3 then
		index = index + 84
	elseif tier == 4 then
		index = index + 126
	elseif tier == 5 then
		index = index + 162
	elseif tier == 6 then
		index = index + 198
	elseif tier == 7 then
		index = index + 234
	elseif tier == 8 then
		index = index + 265
	elseif tier == 9 then
		index = index + 307
	elseif tier == 10 then
		index = index + 349
	elseif tier == 11 then
		index = index + 391
	elseif tier == 12 then
		index = index + 427
	elseif tier == 13 then
		index = index + 463
	elseif tier == 14 then
		index = index + 499
	elseif tier == 15 then
		index = index + 530
	elseif tier == 16 then
		index = index + 535
	elseif tier == 17 then
		index = index + 539
	elseif tier == 18 then
		index = index + 548
	elseif tier == 19 then
		index = index + 557
	elseif tier == 20 then
		index = index + 561
	elseif tier >= 100 then
		index = index
	end
	return index
end

local function GetRecipeSavedVar(index, tier)
	local varDB = (tier == 100) and ESOMRL.SV.fRecipeTrack or ESOMRL.SV.pRecipeTrack
	index = TierIndex(index, tier)
	return varDB[index]
end

local function SetRecipeSavedVar(index, tier, val)
	local varDB = (tier == 100) and ESOMRL.SV.fRecipeTrack or ESOMRL.SV.pRecipeTrack
	index = TierIndex(index, tier)
	varDB[index] = val
end

local function GetIngredientSavedVar(index)
	return ESOMRL.ASV.aIngTrack[index]
end

local function SetIngredientSavedVar(index, val)
	ESOMRL.ASV.aIngTrack[index] = val
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Various utility functions used in other areas.
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function SetTrackedRecipeCountText() -- Updates the current tracked recipe display text.
	ESOMRL_MainFrameSelectionFrameClearSearchLabel:SetText(L.ESOMRL_CLEAR .. ' |cff9900(' .. L.ESOMRL_TRACKING .. '|r |cffffff' .. ESOMRL.SV.cOpts.recipetracklist .. '|r |cff9900' .. L.ESOMRL_RECIPES .. ')|r')
	ESOMRL_MainFrameFurnitureFrameFClearSearchLabel:SetText(L.ESOMRL_CLEAR .. ' |cff9900(' .. L.ESOMRL_TRACKING .. '|r |cffffff' .. ESOMRL.SV.cOpts.furnituretracklist .. '|r |cff9900' .. L.ESOMRL_PATTERNS .. ')|r')
	if StationControl ~= nil then StationControl:RefreshRecipeList() end
end

local function SetTrackedIngredientCountText() -- Updates the current tracked ingredient display text.
	ESOMRL_MainFrameIngredientsFrameClearIngTrackLabel:SetText(L.ESOMRL_ICLEAR .. ' |cff9900(' .. L.ESOMRL_TRACKING .. '|r |cffffff' .. ESOMRL.SV.cOpts.ingtracklist .. '|r |cff9900' .. L.ESOMRL_INGREDIENTS .. ')|r')
end

local function GetItemIdFromLink(itemLink) -- Parse the link data for base itemId.
	local itemId = select(4, ZO_LinkHandler_ParseLink(itemLink))
	return tonumber(itemId)
end

local function RefreshViews() -- Refresh current views when tracking status updates so icons re-populate in real-time.
	ZO_ScrollList_RefreshVisible(ZO_PlayerInventoryBackpack)
	ZO_ScrollList_RefreshVisible(ZO_PlayerBankBackpack)
	ZO_ScrollList_RefreshVisible(ZO_GuildBankBackpack)
	ZO_ScrollList_RefreshVisible(ZO_TradingHouseItemPaneSearchResults)
	ZO_ScrollList_RefreshVisible(ZO_TradingHousePostedItemsList)
end

local function LanguageFormat(langtext) -- ZOS function for removing special characters from names in various languages.
	return zo_strformat("<<t:1>>",langtext)
end

local function LowerSpaceSpecial(text) -- Remove spaces and change to lower case. Also strip all after ':'.
	if GetCVar('Language.2') == 'en' then
		return tostring(text):gsub("%s",''):gsub(":.*",''):gsub("%W",''):lower()
	else
		return zo_strlower(zo_strformat("<<t:1>>",text)):gsub("%s",''):gsub(":.*",''):gsub("%W",'')
	end
end

local function Contains(table, element)	-- Determined if a given element exists in a given table.
	for k,v in pairs(table) do
		if v == element then
			return true
		end
	end
	return false
end

local function FindName(table, element)	-- Returns the table key that contains a given element value.
	for k,v in pairs(table) do	-- Used for determining which key has the name to remove with table.remove.
		if v == element then
			return k
		end
	end
	return 0
end

local function LookupPosition(htable, tval) -- Lookup the itemId's position from a given resource table (fast).
	if htable[tval] then
		return tonumber(htable[tval].position)
	end
	return 0
end

local function GetTextureId(texturePath) -- Returns the selected icon set for the cooking station marks.
	for k, v in pairs(DB.StatOptions) do
		if	v == texturePath then
			return k
		end
	end
	return 0
end

local function SwitchStationIcons(textureId) -- Sets the above station icon texture set to active.
	if textureId == 1 then
		for i = 1, 7 do DB.StationIcons[i] = DB.StatIcons[i] end
	elseif textureId == 2 then
		for i = 1, 7 do local b = i + 7 DB.StationIcons[i] = DB.StatIcons[b] end
	end
end

local function RecountTracked() -- Recount tracked recipes/ingredients once when you log in.
	if ShowMotifs then -- Offset default position if ShowMotifs enabled to match new columns.
		if ESOMRL.ASV.aOpts.bagiconoffset == 25 then
			ESOMRL.ASV.aOpts.bagiconoffset = 82
		end
		if ESOMRL.ASV.aOpts.gstoreiconoffset == -30 then
			ESOMRL.ASV.aOpts.gstoreiconoffset = 27
		end
		if ESOMRL.ASV.aOpts.glistingiconoffset == -30 then
			ESOMRL.ASV.aOpts.glistingiconoffset = 27
		end
		RefreshViews()
	elseif not ShowMotifs then
		if ESOMRL.ASV.aOpts.bagiconoffset == 82 then
			ESOMRL.ASV.aOpts.bagiconoffset = 25
		end
		if ESOMRL.ASV.aOpts.gstoreiconoffset == 27 then
			ESOMRL.ASV.aOpts.gstoreiconoffset = -30
		end
		if ESOMRL.ASV.aOpts.glistingiconoffset == 27 then
			ESOMRL.ASV.aOpts.glistingiconoffset = -30
		end
		RefreshViews()
	end
	ESOMRL.SV.cOpts.recipetracklist = 0
	ESOMRL.SV.cOpts.furnituretracklist = 0
	ESOMRL.SV.cOpts.ingtracklist = 0
	local trackedrecipes = 0
	local trackedpatterns = 0
	local trackedingredients = 0
	local playername = GetUnitName("player")
	for r = 1, pRecipes do
		local savedvar = GetRecipeSavedVar(r, 1)
		if savedvar == 2 or savedvar == 1 then
			trackedrecipes = trackedrecipes + 1
		end
	end
	for i = 1, 50 do
		local savedvar = GetIngredientSavedVar(i)
		if savedvar == 1 then
			trackedingredients = trackedingredients + 1
		end
	end
	for r = 1, fRecipes do
		local savedvar = GetRecipeSavedVar(r, 100)
		if savedvar == 2 or savedvar == 1 then
			trackedpatterns = trackedpatterns + 1
		end
	end
	ESOMRL.SV.cOpts.recipetracklist = trackedrecipes
	ESOMRL.SV.cOpts.furnituretracklist = trackedpatterns
	ESOMRL.SV.cOpts.ingtracklist = trackedingredients
	SetTrackedRecipeCountText()
	SetTrackedIngredientCountText()
	recountTracked = 1
end

local function InitKnown() -- Re-scans for known recipes any time you learn a recipe or open the recipe book.
	local playername = GetUnitName("player")
	for p = 1, pRecipes do
		local savedvar = GetRecipeSavedVar(p, 1)
		if IsItemLinkRecipeKnown(ESOMRL.ProvisioningLinks[p].link) == true then
			if (savedvar == 2) or (savedvar == 1) then
				SetRecipeSavedVar(p, 1, 2)
			else
				SetRecipeSavedVar(p, 1, 3)
			end
			if ESOMRL.SV.cOpts.trackChar == true then
				if not Contains(ESOMRL.ASV.pRecipeKnown[p].names, playername) then
					table.insert(ESOMRL.ASV.pRecipeKnown[p].names, #ESOMRL.ASV.pRecipeKnown[p].names + 1, playername)
				end
			else
				if Contains(ESOMRL.ASV.pRecipeKnown[p].names, playername) then
					table.remove(ESOMRL.ASV.pRecipeKnown[p].names, FindName(ESOMRL.ASV.pRecipeKnown[p].names, playername))
				end
			end
		else
			if savedvar == 2 then
				SetRecipeSavedVar(p, 1, 1)
			elseif savedvar == 3 then
				SetRecipeSavedVar(p, 1, 0)
			end
		end
	end

	for f = 1, fRecipes do
		local savedvar = GetRecipeSavedVar(f, 100)
		if IsItemLinkRecipeKnown(ESOMRL.FurnitureLinks[f].link) == true then
			if (savedvar == 2) or (savedvar == 1) then
				SetRecipeSavedVar(f, 100, 2)
			else
				SetRecipeSavedVar(f, 100, 3)
			end
			if ESOMRL.SV.cOpts.trackChar == true then
				if not Contains(ESOMRL.ASV.fRecipeKnown[f].names, playername) then
					table.insert(ESOMRL.ASV.fRecipeKnown[f].names, #ESOMRL.ASV.fRecipeKnown[f].names + 1, playername)
				end
			else
				if Contains(ESOMRL.ASV.fRecipeKnown[f].names, playername) then
					table.remove(ESOMRL.ASV.fRecipeKnown[f].names, FindName(ESOMRL.ASV.fRecipeKnown[f].names, playername))
				end
			end
		else
			if savedvar == 2 then
				SetRecipeSavedVar(f, 100, 1)
			elseif savedvar == 3 then
				SetRecipeSavedVar(f, 100, 0)
			end
		end
	end

	RefreshViews()
	if recountTracked == 0 then
		RecountTracked()
	end
end

local function NoRecipes() -- Checks if any recipes are known when accessing the cooking station.
	for i = 1, pRecipes do
		if IsItemLinkRecipeKnown(ESOMRL.ProvisioningLinks[i].link) == true then
			return false
		end
	end
	return true
end

local function GetFurnitureSubCat(list) -- Used to get the category of furniture items.
	local catString = ''
	if list > 100 and list < 106 then
		catString = L.ESOMRL_ALCHEMY
	elseif list > 105 and list < 111 then
		catString = L.ESOMRL_BLACKSMITHING
	elseif list > 110 and list < 116 then
		catString = L.ESOMRL_CLOTHING
	elseif list > 115 and list < 121 then
		catString = L.ESOMRL_ENCHANTING
	elseif list > 120 and list < 126 then
		catString = L.ESOMRL_PROVISIONING
	elseif list > 125 and list < 131 then
		catString = L.ESOMRL_WOODWORKING
	elseif list > 130 and list < 136 then
		catString = L.ESOMRL_JEWELRY
	end
	return catString
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Functions to show and hide the recipe list, adjust for laguage settings, and remember position.
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function RestorePosition() -- Restores previous window position when opened.
	local left = ESOMRL.SV.cOpts.xpos
	local top = ESOMRL.SV.cOpts.ypos
	ESOMRL_MainFrame:ClearAnchors()
	ESOMRL_MainFrame:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, left, top)
end

local function FormatTooltipText(text) -- Formats tooltip data by language.
	if GetCVar('Language.2') == 'en' then
		return text
	else
		text = text:gsub('Any Level',L.ESOMRL_ANY)
		text = text:gsub('Level',L.ESOMRL_LEVEL)
		text = text:gsub('Food',L.ESOMRL_FOOD)
		text = text:gsub('Drink',L.ESOMRL_DRINK)
		text = text:gsub('Psijic Ambrosia',LanguageFormat(GetItemLinkName(GetItemLinkRecipeResultItemLink("|H1:item:64223:3:1:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h"))))
		return text
	end
end

local function CloseMain(control, option) -- Closes the main window and returns cursor control.
	if option == 1 then
		InitializeTooltip(InformationTooltip, control, TOPLEFT, 8, -44, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_CLOSE)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		SCENE_MANAGER:ToggleTopLevel(ESOMRL_MainFrame)
	end
end

local function InfoTooltip(option) -- Show the legend.
	if setPage == 1 then
		if option == 1 then
			ESOMRL_MainFrameInfoBox:SetHidden(false)
		elseif option == 2 then
			ESOMRL_MainFrameInfoBox:SetHidden(true)
		end
	elseif setPage == 0 then
		if option == 1 then
			ESOMRL_MainFrameInfoBoxIng:SetHidden(false)
		elseif option == 2 then
			ESOMRL_MainFrameInfoBoxIng:SetHidden(true)
		end
	end
end
		
local function ToggleTooltips(control, option) -- Handles turning list item tooltips on and off.
	if option == 1 then
		InitializeTooltip(InformationTooltip, control, TOPLEFT, 34, -39, BOTTOMRIGHT)
		if ESOMRL.SV.cOpts.lttshow == 1 then
			SetTooltipText(InformationTooltip, L.ESOMRL_LTTHIDE)
		else
			SetTooltipText(InformationTooltip, L.ESOMRL_LTTSHOW)
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if ESOMRL.SV.cOpts.lttshow == 1 then
			ESOMRL.SV.cOpts.lttshow = 0
			ClearTooltip(InformationTooltip)
			InitializeTooltip(InformationTooltip, control, TOPLEFT, 34, -39, BOTTOMRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_LTTSHOW)
		else
			ESOMRL.SV.cOpts.lttshow = 1
			ClearTooltip(InformationTooltip)
			InitializeTooltip(InformationTooltip, control, TOPLEFT, 34, -39, BOTTOMRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_LTTHIDE)
		end
	end
end

local function IngredientPanelSetup() -- Configure the ingredients display when the panel is shown.
	for i = 110, 159 do
		local control = ESOMRL_MainFrameIngredientsFrame:GetChild(i)
		local index = i - 109
		if GetIngredientSavedVar(index) == 1 then
			control:SetText(LanguageFormat(GetItemLinkName(ESOMRL.IngredientLinks[index].link)))
			control:SetColor(1,1,0,1)
			control:SetAlpha(1)
		elseif GetIngredientSavedVar(index) == 0 then
			control:SetText(LanguageFormat(GetItemLinkName(ESOMRL.IngredientLinks[index].link)))
			control:SetColor(0.7647,0.7647,0.7647,1)
			control:SetAlpha(1)
		end
	end
	if ESOMRL.SV.cOpts.destroyjunkingredients == true then
		if ESOMRL.SV.cOpts.destroyunmarkedingredients == 0 then
			ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton1:SetHidden(false)
			ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton2:SetHidden(true)
		elseif ESOMRL.SV.cOpts.destroyunmarkedingredients == 1 then
			ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton1:SetHidden(true)
			ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton2:SetHidden(false)
		end
	elseif ESOMRL.SV.cOpts.destroyjunkingredients == false then
		if ESOMRL.SV.cOpts.junkunmarkedingredients == 0 then
			ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton1:SetHidden(false)
			ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton2:SetHidden(true)
		elseif ESOMRL.SV.cOpts.junkunmarkedingredients == 1 then
			ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton1:SetHidden(true)
			ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton2:SetHidden(false)
		end
	end
end

local function RecipePanelSetup() -- Configure the recipe display when the panel is shown.
	if ESOMRL.SV.cOpts.destroyjunkrecipes == true then
		if ESOMRL.SV.cOpts.destroyunmarkedrecipes == 0 then
			ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton1:SetHidden(false)
			ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton2:SetHidden(true)
		elseif ESOMRL.SV.cOpts.destroyunmarkedrecipes == 1 then
			ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton1:SetHidden(true)
			ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton2:SetHidden(false)
		end
	elseif ESOMRL.SV.cOpts.destroyjunkrecipes == false then
		if ESOMRL.SV.cOpts.junkunmarkedrecipes == 0 then
			ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton1:SetHidden(false)
			ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton2:SetHidden(true)
		elseif ESOMRL.SV.cOpts.junkunmarkedrecipes == 1 then
			ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton1:SetHidden(true)
			ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton2:SetHidden(false)
		end
	end
end

local function RecipeOptionPanel(option) -- Handles turning recipe config panel on and off.
	if option == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
		if ESOMRL.SV.cOpts.recipeconfigpanel == 0 then
			SetTooltipText(InformationTooltip, L.ESOMRL_SRCONFIGPANEL)
		elseif ESOMRL.SV.cOpts.recipeconfigpanel == 1 then
			SetTooltipText(InformationTooltip, L.ESOMRL_HRCONFIGPANEL)
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if ESOMRL.SV.cOpts.recipeconfigpanel == 0 then
			ESOMRL.SV.cOpts.recipeconfigpanel = 1
			ESOMRL_MainFrameListFrameBatchTracking:SetHidden(false)
			ClearTooltip(InformationTooltip)
			InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_HRCONFIGPANEL)
		elseif ESOMRL.SV.cOpts.recipeconfigpanel == 1 then
			ESOMRL.SV.cOpts.recipeconfigpanel = 0
			ESOMRL_MainFrameListFrameBatchTracking:SetHidden(true)
			ClearTooltip(InformationTooltip)
			InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_SRCONFIGPANEL)
		end
		RecipePanelSetup()
	end
end

local function RecipePanelRestore() -- Restore previous status of recipe config panel on load.
	if ESOMRL.SV.cOpts.recipeconfigpanel == 0 then
		ESOMRL_MainFrameListFrameBatchTracking:SetHidden(true)
	elseif ESOMRL.SV.cOpts.recipeconfigpanel == 1 then
		ESOMRL_MainFrameListFrameBatchTracking:SetHidden(false)
	end
	RecipePanelSetup()
end

local function OnCharacterSelected(_, charName, choice) -- Dropbox callback.
	currentChar = charName
end

local function SetupCharacterDropbox() -- Initialize dropbox selection of all tracked characters who know any recipes.
	currentChar = GetUnitName("player")
	table.insert(nameList, 1, currentChar)
	for p = 1, pRecipes do
		if next(ESOMRL.ASV.pRecipeKnown[p].names) ~= nil then
			for n = 1, #ESOMRL.ASV.pRecipeKnown[p].names do
				local name = ESOMRL.ASV.pRecipeKnown[p].names[n]
				if not Contains(nameList, name) then
					table.insert(nameList, #nameList + 1, name)
				end
				if not Contains(dataNames, name) then
					table.insert(dataNames, #dataNames + 1, name)
				end
			end
		end
	end
	for f = 1, fRecipes do
		if next(ESOMRL.ASV.fRecipeKnown[f].names) ~= nil then
			for n = 1, #ESOMRL.ASV.fRecipeKnown[f].names do
				local name = ESOMRL.ASV.fRecipeKnown[f].names[n]
				if not Contains(nameList, name) then
					table.insert(nameList, #nameList + 1, name)
				end
				if not Contains(dataNames, name) then
					table.insert(dataNames, #dataNames + 1, name)
				end
			end
		end
	end
	charnameDropdown = WINDOW_MANAGER:CreateControlFromVirtual('CharacterDropdownList', ESOMRL_MainFrameListFrameBatchTracking, 'ZO_StatsDropdownRow')
	charnameDropdown:SetWidth(300)
	charnameDropdown:SetAnchor(TOPLEFT, ESOMRL_MainFrameListFrameBatchTracking, TOPLEFT, -4, 2)
	charnameDropdown:GetNamedChild('Dropdown'):SetWidth(295)
	charnameDropdown.dropdown:SetSelectedItem(currentChar)
	for k,v in pairs(nameList) do
		local entry = charnameDropdown.dropdown:CreateItemEntry(v, OnCharacterSelected)
		charnameDropdown.dropdown:AddItem(entry)
	end
end

local function RemoveCharacter(option) -- Handles removing selected character from the tracking database.
	if option == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameCloseButton, TOPLEFT, 8, -44, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_REMOVECHAR)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if currentChar ~= GetUnitName("player") then
			table.remove(nameList, FindName(nameList, currentChar)) -- Remove the selected character from the current name table.
			for p = 1, pRecipes do -- Remove the selected character from the global tracking database.
				if Contains(ESOMRL.ASV.pRecipeKnown[p].names, currentChar) then
					table.remove(ESOMRL.ASV.pRecipeKnown[p].names, FindName(ESOMRL.ASV.pRecipeKnown[p].names, currentChar))
				end
			end
			for f = 1, fRecipes do
				if Contains(ESOMRL.ASV.fRecipeKnown[f].names, currentChar) then
					table.remove(ESOMRL.ASV.fRecipeKnown[f].names, FindName(ESOMRL.ASV.fRecipeKnown[f].names, currentChar))
				end
			end
			currentChar = GetUnitName("player") -- Set the addon-global active name to the current character.
			charnameDropdown.dropdown:ClearItems() -- Clear the list of names in the dropdown menu so we can rebuild.
			for k,v in pairs(nameList) do -- Rebuild using the names table we just removed a character from.
				local entry = charnameDropdown.dropdown:CreateItemEntry(v, OnCharacterSelected)
				charnameDropdown.dropdown:AddItem(entry)
			end
			charnameDropdown.dropdown:SetSelectedItem(currentChar) -- Set the dropdown selected item to the current character.
		end
	end
end

local function SwitchPage(control, option) -- Handles switching between recipe and ingredient pages.
	if option == 1 then
		if setPage == 1 then
			InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, -53, -4, TOPRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_SHOWING)
		elseif setPage == 0 then
			InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, -53, -4, TOPRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_SHOWRECIPE)
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if setPage == 1 then
			ClearTooltip(InformationTooltip)
		--	InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, -53, -4, TOPRIGHT)
		--	SetTooltipText(InformationTooltip, L.ESOMRL_SHOWRECIPE)
			ESOMRL_MainFrameSelectionFrame:SetHidden(true)
			ESOMRL_MainFrameFurnitureFrame:SetHidden(true)
			ESOMRL_MainFrameListFrame:SetHidden(true)
			ESOMRL_MainFrameIngredientsFrame:SetHidden(false)
			ESOMRL_MainFrameFoodButton:SetHidden(true)
			ESOMRL_MainFrameFurnitureButton:SetHidden(true)
			ESOMRL_MainFramePageButton:SetHidden(false)
			ESOMRL_MainFramePageButton:ClearAnchors()
			ESOMRL_MainFramePageButton:SetAnchor(TOPLEFT, ESOMRL_MainFrame, TOPLEFT, -7, -6)
			SetTrackedIngredientCountText()
			IngredientPanelSetup()
			setPage = 0
		elseif setPage == 0 then
			ClearTooltip(InformationTooltip)
		--	InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, -53, -4, TOPRIGHT)
		--	SetTooltipText(InformationTooltip, L.ESOMRL_SHOWING)
			ESOMRL_MainFrameSelectionFrame:SetHidden(false)
			ESOMRL_MainFrameFurnitureFrame:SetHidden(true)
			ESOMRL_MainFrameListFrame:SetHidden(false)
			ESOMRL_MainFrameIngredientsFrame:SetHidden(true)
			ESOMRL_MainFrameFoodButton:SetHidden(true)
			ESOMRL_MainFrameFurnitureButton:SetHidden(false)
			ESOMRL_MainFramePageButton:SetHidden(false)
			ESOMRL_MainFramePageButton:ClearAnchors()
			ESOMRL_MainFramePageButton:SetAnchor(TOPLEFT, ESOMRL_MainFrameFurnitureButton, TOPRIGHT, 0, 0)
			ESOMRL.XMLNavigation(304, nil, nil, 1)
			setPage = 1
		end
		setMode = 1
	end
end

local function SwitchMode(control, option) -- Handles switching between recipe and furniture modes.
	if option == 1 then
		if setMode == 1 then
			InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, -53, -4, TOPRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_SHOWFURNITURE)
		elseif setMode == 0 then
			InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, -53, -4, TOPRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_SHOWRECIPE)
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if setPage == 1 then
			ClearTooltip(InformationTooltip)
		--	InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, -53, -4, TOPRIGHT)
		--	SetTooltipText(InformationTooltip, L.ESOMRL_SHOWRECIPE)
			ESOMRL_MainFrameSelectionFrame:SetHidden(true)
			ESOMRL_MainFrameFurnitureFrame:SetHidden(false)
			ESOMRL_MainFrameListFrame:SetHidden(false)
			ESOMRL_MainFrameIngredientsFrame:SetHidden(true)
			ESOMRL_MainFrameFoodButton:SetHidden(false)
			ESOMRL_MainFrameFurnitureButton:SetHidden(true)
			ESOMRL_MainFramePageButton:SetHidden(true)
			
			ESOMRL_MainFrameListFrameQuality:SetHidden(true)
			
			
			setPage = 0
			setMode = 0
			ESOMRL.XMLNavigation(304, nil, nil, 102)
		elseif setPage == 0 then
			ClearTooltip(InformationTooltip)
		--	InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, -53, -4, TOPRIGHT)
		--	SetTooltipText(InformationTooltip, L.ESOMRL_SHOWFURNITURE)
			ESOMRL_MainFrameSelectionFrame:SetHidden(false)
			ESOMRL_MainFrameFurnitureFrame:SetHidden(true)
			ESOMRL_MainFrameListFrame:SetHidden(false)
			ESOMRL_MainFrameIngredientsFrame:SetHidden(true)
			ESOMRL_MainFrameFoodButton:SetHidden(true)
			ESOMRL_MainFrameFurnitureButton:SetHidden(false)
			ESOMRL_MainFramePageButton:SetHidden(false)
			
			
			ESOMRL_MainFrameListFrameQuality:SetHidden(false)
			
			
			setPage = 1
			setMode = 1
			ESOMRL.XMLNavigation(304, nil, nil, 1)
		end
	end
end

local function WritButton(control, option) -- Button tooltip for writ lookups.
	if option == 1 then
		InitializeTooltip(InformationTooltip, control, TOPLEFT, 0, 0, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_FINDWRIT)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	end
end

local function OnMoveStop() -- Saves the current window position when closed.
	ESOMRL.SV.cOpts.xpos = ESOMRL_MainFrame:GetLeft()
	ESOMRL.SV.cOpts.ypos = ESOMRL_MainFrame:GetTop()
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Functions related to the quest-event-based tracking of writ requirements.
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function CheckForWritMatch(conditionText)
	local function SetWritIngredients(sm)
		local ingredients = {}
		local linkcheck = ESOMRL.ProvisioningLinks[sm].link
		local numIngredients = GetItemLinkRecipeNumIngredients(linkcheck)
		for i = 1, 50 do
			local namelookup = LowerSpaceSpecial(GetItemLinkName(ESOMRL.IngredientLinks[i].link))
			table.insert(ingredients, #ingredients + 1, namelookup)
		end
		for n = 1, numIngredients do
			local ingredientName,_ = GetItemLinkRecipeIngredientInfo(linkcheck, n)
			ingredientName = LowerSpaceSpecial(ingredientName)
			local ingID = tonumber(ESOMRL.IngredientLinks[tonumber(FindName(ingredients, ingredientName))].id)
			if not Contains(WritIng, ingID) then
				table.insert(WritIng, #WritIng + 1, ingID)
			end
		end
	end
	local mTable = {}
	local wWords = {}
	for w in conditionText:gmatch("%S+") do table.insert(wWords, #wWords + 1, LowerSpaceSpecial(w)) end
	local text = LowerSpaceSpecial(conditionText)
	local mCount = 0
	for i = 1, pRecipes do
		local search = LowerSpaceSpecial(GetItemLinkName(ESOMRL.ProvisioningLinks[i].item))
		local name = zo_strlower(LanguageFormat(GetItemLinkName(ESOMRL.ProvisioningLinks[i].item)))
		if (string.find(text,search) ~= nil) then
			table.insert(mTable, #mTable + 1, {search=name, pos=i}) mCount = mCount + 1
		end
	end
	if mCount > 0 then
		if mCount > 1 then
			local smCount = 0
			local sm = 0
			for i = 1, mCount do
				local tsmCount = 0
				local sWords = {}
				local sString = mTable[i].search
				for w in sString:gmatch("%S+") do table.insert(sWords, #sWords + 1, LowerSpaceSpecial(w)) end
				for m = 1, #sWords do
					for w = 1, #wWords do
						if sWords[m] == wWords[w] then tsmCount = tsmCount + 1 end
					end
					if tsmCount > smCount then
						smCount = tsmCount
						sm = mTable[i].pos
					end
				end
			end
			SetWritIngredients(sm)
			return GetItemIdFromLink(ESOMRL.ProvisioningLinks[sm].item)
		elseif mCount == 1 then
			SetWritIngredients(mTable[1].pos)
			return GetItemIdFromLink(ESOMRL.ProvisioningLinks[mTable[1].pos].item)
		end
	else
		return 0
	end
end

local function GetWritID()
	for i = 1, MAX_JOURNAL_QUESTS do
		if IsValidQuestIndex(i) then
			local questType = GetJournalQuestType(i)
			if questType == 4 then
				local QuestName = GetJournalQuestName(i)
				if QuestName == L.ESOMRL_WRITQUEST then
					return i
				end
			end
		end
	end
	return 0
end

function StationWritCheck()
	local writIndex = GetWritID()
	local questTable = {}
	DB.WritQuest.reqId1 = 0
	DB.WritQuest.reqId2 = 0
	WritIng = {}
	local function CheckInfo(q,r)
		local a, b, _, _, _, _ = GetJournalQuestConditionInfo(q,1,r)
		return a, b
	end
	if writIndex == 0 then
		DB.WritQuest.reqId1 = 0
		DB.WritQuest.reqId2 = 0
		return
	end
	for i = 1, GetJournalQuestNumConditions(writIndex) do
		local conditionText, current = CheckInfo(writIndex,i)
		table.insert(questTable, #questTable + 1, {conditionText=conditionText, current=current})
	end
	for r = 1, #questTable do
		if string.find(questTable[r].conditionText, L.ESOMRL_WRITFINISH) then
			DB.WritQuest.reqId1 = 0
			DB.WritQuest.reqId2 = 0
			return
		end
		if questTable[r].conditionText ~= "" and questTable[r].current == 0 then
			if DB.WritQuest.reqId1 == 0 then
				DB.WritQuest.reqId1 = CheckForWritMatch(questTable[r].conditionText)
			elseif DB.WritQuest.reqId2 == 0 then
				DB.WritQuest.reqId2 = CheckForWritMatch(questTable[r].conditionText)
			end
		end
	end
end

local function OnQuestAdded(eventCode, journalIndex, questName, objectiveName)
	if questName == L.ESOMRL_WRITQUEST then
		StationWritCheck()
	end
end

local function OnQuestConditionCounterChanged(eventCode, journalIndex, questName, conditionText, conditionType, currConditionVal, newConditionVal, conditionMax, isFailCondition, stepOverrideText, isPushed, isComplete, isConditionComplete, isStepHidden)
	if questName == L.ESOMRL_WRITQUEST then
		local a, b, _, _, _, _ = GetJournalQuestConditionInfo(journalIndex,1,1)
		local c, d, _, _, _, _ = GetJournalQuestConditionInfo(journalIndex,1,2)
		local e, f, _, _, _, _ = GetJournalQuestConditionInfo(journalIndex,1,2)
		if string.find(a, "L.ESOMRL_WRITFINISH") or string.find(c, "L.ESOMRL_WRITFINISH") or string.find(e, "L.ESOMRL_WRITFINISH") then
			DB.WritQuest.reqId1 = 0
			DB.WritQuest.reqId2 = 0
			return
		elseif a == "" and c == "" then
			DB.WritQuest.reqId1 = 0
			DB.WritQuest.reqId2 = 0
			return
		else
			StationWritCheck()
		end
	end
end

local function OnQuestComplete(eventCode, questName, level, previousExperience, currentExperience, rank, previousPoints, currentPoints)
	if questName == L.ESOMRL_WRITQUEST then
		DB.WritQuest.reqId1 = 0 DB.WritQuest.reqId2 = 0 return
	end
end

local function OnQuestRemoved(eventCode, isCompleted, journalIndex, questName, zoneIndex, poiIndex)
	if questName == L.ESOMRL_WRITQUEST then
		DB.WritQuest.reqId1 = 0 DB.WritQuest.reqId2 = 0 return
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Set up hooks into various tooltips and add appropriate text.
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function AddRecipeTooltipLine(control, itemId, itemLink, option, furniture) -- Modifies tooltip based on various options.
	local pos = 0
	local resLink
	local saved
	local knownDB

	if option == 1 then
		if furniture == 0 then
			pos = LookupPosition(ESOMRL.RecipeMaterials, itemId)
			resLink = ESOMRL.RecipeMaterials[itemId].item
		else
			pos = LookupPosition(ESOMRL.RecipeFMaterials, itemId)
			resLink = ESOMRL.RecipeFMaterials[itemId].item
		end
	elseif option == 0 then
		if furniture == 0 then
			pos = LookupPosition(ESOMRL.ResultMaterials, itemId)
			resLink = ESOMRL.ProvisioningLinks[tonumber(ESOMRL.ResultMaterials[itemId].position)].item
		else
			pos = LookupPosition(ESOMRL.ResultFMaterials, itemId)
			resLink = ESOMRL.FurnitureLinks[tonumber(ESOMRL.ResultFMaterials[itemId].position)].item
		end
	end

	if furniture == 0 then
		saved = GetRecipeSavedVar(pos, 0)
		knownDB = ESOMRL.ASV.pRecipeKnown
	else
		saved = GetRecipeSavedVar(pos, 100)
		knownDB = ESOMRL.ASV.fRecipeKnown
	end

	if pos ~= 0 then
		if ESOMRL.ASV.aOpts.ingfood == true then -- Show ingredients in food item tooltips (if enabled in global options).
			if option == 0 then
				local inglink
				if furniture == 0 then
					inglink = ESOMRL.ProvisioningLinks[tonumber(ESOMRL.ResultMaterials[itemId].position)].link
				else
					inglink = ESOMRL.FurnitureLinks[tonumber(ESOMRL.ResultFMaterials[itemId].position)].link
				end
			
				local ingnumber = GetItemLinkRecipeNumIngredients(inglink)
				local ingstring = ''
				for i = 1, ingnumber do
					local ing, num, req = GetItemLinkRecipeIngredientInfo(inglink, i)
					if i == ingnumber then
						if num > 0 then
							ingstring = ingstring .. '|cffffff' .. tostring(LanguageFormat(ing)) .. " \(" .. req .. "\)" .. '|r'
						else
							ingstring = ingstring .. '|c656565' .. tostring(LanguageFormat(ing))  .. " \(" .. req .. "\)" .. '|r'
						end
					else
						if num > 0 then
							ingstring = ingstring .. '|cffffff' .. tostring(LanguageFormat(ing))  .. " \(" .. req .. "\)" .. '|r' .. ', '
						else
							ingstring = ingstring .. '|c656565' .. tostring(LanguageFormat(ing))  .. " \(" .. req .. "\)" .. '|r' .. ', '
						end
					end
				end
				control:AddVerticalPadding(20)
				control:AddLine(L.ESOMRL_INGFOOD, 'ZoFontHeader', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
				control:AddVerticalPadding(-10)
				control:AddLine(ingstring, 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
			end
		end

		if ESOMRL.ASV.aOpts.known == true then -- Show list of characters who know/are able to craft the given recipe or food item in tooltips.
			local namestring
			local cstring
			if next(knownDB[pos].names) ~= nil then
				control:AddVerticalPadding(10)
				ZO_Tooltip_AddDivider(control)
				control:AddVerticalPadding(10)
				if option == 1 then
					control:AddLine(L.ESOMRL_KNOWN, 'ZoFontHeader', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
				elseif option == 0 then
					control:AddLine(L.ESOMRL_CRAFTABLE, 'ZoFontHeader', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
				end
				local count = #knownDB[pos].names
				for i = 1, count do
					local name = knownDB[pos].names[i]
					if i == 1 then
						if count == i then
							control:AddVerticalPadding(-10)
							control:AddLine('|c66ccff' .. name .. '|r', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
						else
							if #name > 35 then
								control:AddVerticalPadding(-10)
								control:AddLine('|c66ccff' .. name .. '|r' .. ', ', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
							else
								namestring = '|c66ccff' .. name .. '|r' .. ', '
								cstring = name .. ', '
							end
						end
					else
						if count == i then
							local tempa = namestring .. '|c66ccff' .. name .. '|r'
							local tempb = cstring .. name
							if #tempb > 45 then
								control:AddVerticalPadding(-10)
								control:AddLine(namestring, 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
								control:AddVerticalPadding(-10)
								control:AddLine('|c66ccff' .. name .. '|r', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
							else
								control:AddVerticalPadding(-10)
								control:AddLine(tempa, 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
							end
						else
							if #name > 35 then
								control:AddVerticalPadding(-10)
								control:AddLine('|c66ccff' .. name .. '|r' .. ', ', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
							else
								local tempa = namestring .. '|c66ccff' .. name .. '|r' .. ', '
								local tempb = cstring .. name .. ','
								if #tempb > 45 then
									control:AddVerticalPadding(-10)
									control:AddLine(namestring, 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, LEFT, false)
									namestring = '|c66ccff' .. name .. '|r' .. ', '
									cstring = name .. ', '
								else
									namestring = tempa
									cstring = tempb
								end
							end
						end
					end
				end
			end
		end
	end

	if option == 1 then -- Show if selected recipe/food item is needed for current writ (if any).
		if IsItemLinkRecipeKnown(itemLink) == false then
			if saved == 1 then
				control:AddVerticalPadding(20)
				control:AddLine('|cfee854' .. L.ESOMRL_RTRACK .. '|r', 'ZoFontHeader2', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
				if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
					control:AddLine('|cffff00' .. L.ESOMRL_NWRITU .. '|r', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
				end
			elseif saved == 0 then
				if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
					control:AddVerticalPadding(20)
					control:AddLine('|cffff00' .. L.ESOMRL_NWRITU .. '|r', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
				end
			end
		elseif IsItemLinkRecipeKnown(itemLink) == true then
			if saved == 2 then
				control:AddVerticalPadding(20)
				control:AddLine('|cfee854' .. L.ESOMRL_RTRACK .. '|r', 'ZoFontHeader2', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
				if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
					control:AddLine('|cffff00' .. L.ESOMRL_NWRITK .. '|r', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
				end
			elseif saved == 3 then
				if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
					control:AddVerticalPadding(20)
					control:AddLine('|cffff00' .. L.ESOMRL_NWRITK .. '|r', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
				end
			end
		end
	elseif option == 0 then
		if pos ~= 0 then
			if saved == 1 or saved == 2 then
				if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
					control:AddVerticalPadding(20)
					control:AddLine('|cffff00' .. L.ESOMRL_NWRIT .. '|r', 'ZoFontHeader2', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
				end
			elseif saved == 0 or saved == 3 then
				if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
					control:AddVerticalPadding(20)
					control:AddLine('|cffff00' .. L.ESOMRL_NWRIT .. '|r', 'ZoFontHeader2', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
				end
			end
		end
	end
end

local function AddIngredientTooltipLine(control, itemId) -- Modifies ingredient tooltip with tracked status.
	local pos = LookupPosition(ESOMRL.IngredientMaterials, itemId)
	if pos ~= 0 then
		if GetIngredientSavedVar(pos) == 1 then
			control:AddVerticalPadding(30)
			control:AddLine('|cfee854' .. L.ESOMRL_ITRACK .. '|r', 'ZoFontHeader2', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
			if Contains(WritIng, itemId) then
				control:AddLine('|cffff00' .. L.ESOMRL_NWRIT .. '|r', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
			end
		else
			if Contains(WritIng, itemId) then
				control:AddVerticalPadding(30)
				control:AddLine('|cffff00' .. L.ESOMRL_NWRIT .. '|r', 'ZoFontGame', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
			end
		end
	end
end

local function HookTooltips()
	local ESOMRLSetBagItemTooltip = ItemTooltip.SetBagItem
	ItemTooltip.SetBagItem = function(control, bagId, slotIndex)
		local itemLink = GetItemLink(bagId, slotIndex)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetBagItemTooltip(control, bagId, slotIndex)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
	local ESOMRLSetLootItemTooltip = ItemTooltip.SetLootItem
	ItemTooltip.SetLootItem = function(control, lootId)
		local itemLink = GetLootItemLink(lootId)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetLootItemTooltip(control, lootId)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
	local ESOMRLSetTradingHouseItemTooltip = ItemTooltip.SetTradingHouseItem
	ItemTooltip.SetTradingHouseItem = function(control, tradingHouseIndex)
		local itemLink = GetTradingHouseSearchResultItemLink(tradingHouseIndex)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetTradingHouseItemTooltip(control, tradingHouseIndex)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
	local ESOMRLSetTradingHouseListingTooltip = ItemTooltip.SetTradingHouseListing
	ItemTooltip.SetTradingHouseListing = function(control, tradingHouseListingIndex)
		local itemLink = GetTradingHouseListingItemLink(tradingHouseListingIndex)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetTradingHouseListingTooltip(control, tradingHouseListingIndex)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
	local ESOMRLSetAttachedMailItemTooltip = ItemTooltip.SetAttachedMailItem
	ItemTooltip.SetAttachedMailItem = function(control, mailId, attachmentIndex)
		local itemLink = GetAttachedItemLink(mailId, attachmentIndex)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetAttachedMailItemTooltip(control, mailId, attachmentIndex)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
	local ESOMRLSetBuybackTooltip = ItemTooltip.SetBuybackItem
	ItemTooltip.SetBuybackItem = function(control, slotIndex)
		local itemLink = GetBuybackItemLink(slotIndex)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetBuybackTooltip(control, slotIndex)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
	local ESOMRLSetTradeItemTooltip = ItemTooltip.SetTradeItem
	ItemTooltip.SetTradeItem = function(control, tradeId, slotIndex)
		local itemLink = GetTradeItemLink(slotIndex)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetTradeItemTooltip(control, tradeId, slotIndex)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
	local ESOMRLSetStoreItemTooltip = ItemTooltip.SetStoreItem
	ItemTooltip.SetStoreItem = function(control, slotIndex)
		local itemLink = GetStoreItemLink(slotIndex)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetStoreItemTooltip(control, slotIndex)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
	local ESOMRLSetLinkTooltip = PopupTooltip.SetLink
	PopupTooltip.SetLink = function(control, itemLink)
		local itemId = GetItemIdFromLink(itemLink)
		ESOMRLSetLinkTooltip(control, itemLink)
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			AddIngredientTooltipLine(control, itemId)
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 0)
		elseif ESOMRL.ResultMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 0)
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 1, 1)
		elseif ESOMRL.ResultFMaterials[itemId] ~= nil then
			AddRecipeTooltipLine(control, itemId, itemLink, 0, 1)
		end
	end
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Set up hooks into various inventories and apply the tracking icon if enabled.
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function CreateInventoryTrackingControl(parent)
	local control = WINDOW_MANAGER:CreateControl(parent:GetName() .. 'InventoryTrackingControl', parent, CT_TEXTURE)
	control:SetDrawTier(DT_HIGH)
	control:SetHidden(true)
	return control
end

local function CreateInventoryKnownControl(parent)
	local control = WINDOW_MANAGER:CreateControl(parent:GetName() .. 'InventoryKnownControl', parent, CT_TEXTURE)
	control:SetDrawTier(DT_HIGH)
	control:SetHidden(true)
	return control
end

local function AddTrackingIndicator(control, bagID, slotIndex, itemLink, relativePoint, opt)
	local TrackingControl = control:GetNamedChild('InventoryTrackingControl')
	local KnownControl = control:GetNamedChild('InventoryKnownControl')
	local itemType = GetItemLinkItemType(itemLink)
	local itemId = GetItemIdFromLink(itemLink)
	local playerChar = GetUnitName("player")
	local IsGridViewEnabled
	
	if not ESOMRL.ASV.aOpts.inventoryicons then
		if TrackingControl then TrackingControl:SetHidden(true) end
		if KnownControl then KnownControl:SetHidden(true) end
		return
	end
	
	if not TrackingControl then TrackingControl = CreateInventoryTrackingControl(control) else TrackingControl:SetHidden(true) end
	if not KnownControl then KnownControl = CreateInventoryKnownControl(control) else KnownControl:SetHidden(true) end

	if itemType ~= ITEMTYPE_INGREDIENT and itemType ~= ITEMTYPE_RECIPE and itemType ~= ITEMTYPE_FOOD and itemType ~= ITEMTYPE_DRINK and itemType ~= ITEMTYPE_FURNISHING then
		TrackingControl:SetHidden(true)
		KnownControl:SetHidden(true)
		return
	end
	if itemType == ITEMTYPE_INGREDIENT and ESOMRL.IngredientMaterials[itemId] == nil then
		TrackingControl:SetHidden(true)
		KnownControl:SetHidden(true)
		return
	end
	if itemType == ITEMTYPE_RECIPE and ESOMRL.RecipeMaterials[itemId] == nil and ESOMRL.RecipeFMaterials[itemId] == nil then
		TrackingControl:SetHidden(true)
		KnownControl:SetHidden(true)
		return
	end
	if itemType == ITEMTYPE_FOOD and ESOMRL.ResultMaterials[itemId] == nil then
		TrackingControl:SetHidden(true)
		KnownControl:SetHidden(true)
		return
	end
	if itemType == ITEMTYPE_DRINK and ESOMRL.ResultMaterials[itemId] == nil then
		TrackingControl:SetHidden(true)
		KnownControl:SetHidden(true)
		return
	end
	if itemType == ITEMTYPE_FURNISHING and ESOMRL.ResultFMaterials[itemId] == nil then
		TrackingControl:SetHidden(true)
		KnownControl:SetHidden(true)
		return
	end

	if ( control.isGrid or ( control:GetWidth() - control:GetHeight() < 5 ) ) then
		IsGridViewEnabled = true else IsGridViewEnabled = false
	end

	local controlName = WINDOW_MANAGER:GetControlByName(control:GetName() .. 'Name')
	TrackingControl:ClearAnchors()

	if opt == 2 then
		TrackingControl:SetAnchor(LEFT, controlName, relativePoint, ESOMRL.ASV.aOpts.gstoreiconoffset, 9)
		KnownControl:SetAnchor(LEFT, controlName, relativePoint, ESOMRL.ASV.aOpts.gstoreiconoffset, 9)
	elseif opt == 3 then
		TrackingControl:SetAnchor(LEFT, controlName, relativePoint, ESOMRL.ASV.aOpts.glistingiconoffset, 0)
		KnownControl:SetAnchor(LEFT, controlName, relativePoint, ESOMRL.ASV.aOpts.glistingiconoffset, 0)
	elseif opt == 1 then
		if IsGridViewEnabled then
			TrackingControl:SetAnchor(LEFT, parent, BOTTOMLEFT, 4, -12)
			KnownControl:SetAnchor(LEFT, parent, BOTTOMLEFT, 4, -12)
		else
			TrackingControl:SetAnchor(LEFT, controlName, relativePoint, ESOMRL.ASV.aOpts.bagiconoffset)
			KnownControl:SetAnchor(LEFT, controlName, relativePoint, ESOMRL.ASV.aOpts.bagiconoffset)
		end
	end

	if itemType == ITEMTYPE_INGREDIENT then
		local saved = GetIngredientSavedVar(tonumber(ESOMRL.IngredientMaterials[itemId].position))
		if KnownControl then KnownControl:SetHidden(true) end
		if saved == 0 then
			if Contains(WritIng, itemId) then
				if IsGridViewEnabled then
					TrackingControl:SetDimensions(14, 14)
					TrackingControl:SetTexture('/MasterRecipeList/bin/trackings_igv.dds')
					TrackingControl:SetHidden(false)
					KnownControl:SetHidden(true)
				else
					TrackingControl:SetDimensions(20, 20)
					TrackingControl:SetTexture('/MasterRecipeList/bin/trackings.dds')
					TrackingControl:SetHidden(false)
					KnownControl:SetHidden(true)
				end
			else			
				if TrackingControl then TrackingControl:SetHidden(true) end
			end
		else
			if IsGridViewEnabled then
				TrackingControl:SetDimensions(14, 14)
				if Contains(WritIng, itemId) then
					TrackingControl:SetTexture('/MasterRecipeList/bin/trackings_igv.dds')
				else
					TrackingControl:SetTexture('/MasterRecipeList/bin/tracking_igv.dds')
				end
				TrackingControl:SetHidden(false)
				KnownControl:SetHidden(true)
			else
				TrackingControl:SetDimensions(20, 20)
				if Contains(WritIng, itemId) then
					TrackingControl:SetTexture('/MasterRecipeList/bin/trackings.dds')
				else
					TrackingControl:SetTexture('/MasterRecipeList/bin/tracking.dds')
				end
				TrackingControl:SetHidden(false)
				KnownControl:SetHidden(true)
			end
		end
	elseif itemType == ITEMTYPE_RECIPE then
		local pos
		local saved
		local resLink
		local knownDB
		if ESOMRL.RecipeMaterials[itemId] ~= nil then
			pos = tonumber(ESOMRL.RecipeMaterials[itemId].position)
			saved = GetRecipeSavedVar(tonumber(ESOMRL.RecipeMaterials[itemId].position), 1)
			resLink = ESOMRL.RecipeMaterials[itemId].item
			knownDB = ESOMRL.ASV.pRecipeKnown
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			pos = tonumber(ESOMRL.RecipeFMaterials[itemId].position)
			saved = GetRecipeSavedVar(tonumber(ESOMRL.RecipeFMaterials[itemId].position), 100)
			resLink = ESOMRL.RecipeFMaterials[itemId].item
			knownDB = ESOMRL.ASV.fRecipeKnown
		else
			pos = 0
		end

		if pos == 0 then
			if TrackingControl then TrackingControl:SetHidden(true) end
			if KnownControl then KnownControl:SetHidden(true) end
		else
			if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
				if IsGridViewEnabled then
					TrackingControl:SetDimensions(14, 14)
					TrackingControl:SetTexture('/MasterRecipeList/bin/trackings_igv.dds')
					TrackingControl:SetHidden(false)
				else
					TrackingControl:SetDimensions(20, 20)
					TrackingControl:SetTexture('/MasterRecipeList/bin/trackings.dds')
					TrackingControl:SetHidden(false)
				end
			elseif (saved ~= 1) and (saved ~= 2) then
				if TrackingControl then TrackingControl:SetHidden(true) end
				if ESOMRL.SV.cOpts.tchart then
					if not Contains(knownDB[pos].names, nameList[ESOMRL.SV.cOpts.trackingchar]) and not Contains(knownDB[pos].names, nameList[ESOMRL.SV.cOpts.rtrackingchar]) then
						if IsGridViewEnabled then
							KnownControl:SetDimensions(14, 14)
							KnownControl:SetTexture('/MasterRecipeList/bin/trackcknown_igv.dds')
							KnownControl:SetHidden(false)
						else
							KnownControl:SetDimensions(20, 20)
							KnownControl:SetTexture('/MasterRecipeList/bin/trackcknown.dds')
							KnownControl:SetHidden(false)
						end
					elseif ESOMRL.SV.cOpts.altknown then
						if ESOMRL.SV.cOpts.altknownself then
							if Contains(knownDB[pos].names, playerChar) then
								if IsGridViewEnabled then
									KnownControl:SetDimensions(14, 14)
									KnownControl:SetTexture('/MasterRecipeList/bin/trackaknown_igv.dds')
									KnownControl:SetHidden(false)
								else
									KnownControl:SetDimensions(20, 20)
									KnownControl:SetTexture('/MasterRecipeList/bin/trackaknown.dds')
									KnownControl:SetHidden(false)
								end
							end
						elseif not ESOMRL.SV.cOpts.altknownself then
							if Contains(knownDB[pos].names, nameList[ESOMRL.SV.cOpts.trackingchar]) or Contains(knownDB[pos].names, nameList[ESOMRL.SV.cOpts.rtrackingchar]) then
								if IsGridViewEnabled then
									KnownControl:SetDimensions(14, 14)
									KnownControl:SetTexture('/MasterRecipeList/bin/trackaknown_igv.dds')
									KnownControl:SetHidden(false)
								else
									KnownControl:SetDimensions(20, 20)
									KnownControl:SetTexture('/MasterRecipeList/bin/trackaknown.dds')
									KnownControl:SetHidden(false)
								end
							end
						end
					end
				end
			else
				if IsGridViewEnabled then
					if KnownControl then KnownControl:SetHidden(true) end
					TrackingControl:SetDimensions(14, 14)
					TrackingControl:SetTexture('/MasterRecipeList/bin/tracking_igv.dds')
					TrackingControl:SetHidden(false)
				else
					if KnownControl then KnownControl:SetHidden(true) end
					TrackingControl:SetDimensions(20, 20)
					TrackingControl:SetTexture('/MasterRecipeList/bin/tracking.dds')
					TrackingControl:SetHidden(false)
				end
			end
		end
	elseif itemType == ITEMTYPE_FOOD or itemType == ITEMTYPE_DRINK then
		local resLink = ESOMRL.ProvisioningLinks[tonumber(ESOMRL.ResultMaterials[itemId].position)].item
		if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
			if IsGridViewEnabled then
				TrackingControl:SetDimensions(14, 14)
				TrackingControl:SetTexture('/MasterRecipeList/bin/trackings_igv.dds')
				TrackingControl:SetHidden(false)
			else
				TrackingControl:SetDimensions(20, 20)
				TrackingControl:SetTexture('/MasterRecipeList/bin/trackings.dds')
				TrackingControl:SetHidden(false)
			end
		end
	else
		if TrackingControl then TrackingControl:SetHidden(true) end
		if KnownControl then KnownControl:SetHidden(true) end
	end

	if FCOIS and ESOMRL.SV.cOpts.fcoitemsaver == true then -- Compatibility for FCO ItemSaver
		if itemType == ITEMTYPE_INGREDIENT and ESOMRL.IngredientMaterials[itemId] ~= nil then
			local saved = GetIngredientSavedVar(tonumber(ESOMRL.IngredientMaterials[itemId].position))
			if saved ~= 0 then
				FCOIS.MarkItem(bagID, slotIndex, 1, true, true)
			end
		elseif itemType == ITEMTYPE_RECIPE and ESOMRL.RecipeMaterials[itemId] ~= nil then
			local saved = GetRecipeSavedVar(tonumber(ESOMRL.RecipeMaterials[itemId].position), 1)
			if (saved == 1) or (saved == 2) then
				FCOIS.MarkItem(bagID, slotIndex, 1, true, true)
			end
		elseif itemType == ITEMTYPE_RECIPE and ESOMRL.RecipeFMaterials[itemId] ~= nil then
			local saved = GetRecipeSavedVar(tonumber(ESOMRL.RecipeFMaterials[itemId].position), 100)
			if (saved == 1) or (saved == 2) then
				FCOIS.MarkItem(bagID, slotIndex, 1, true, true)
			end
		end
	end
end

local function HookBags()
	for k,v in pairs(PLAYER_INVENTORY.inventories) do
		local listView = v.listView
		if ( listView and listView.dataTypes and listView.dataTypes[1] ) then
			ZO_PreHook(listView.dataTypes[1], "setupCallback", function(control, slot)
				local itemLink = GetItemLink(control.dataEntry.data.bagId, control.dataEntry.data.slotIndex, LINK_STYLE_BRACKETS)
				AddTrackingIndicator(control, control.dataEntry.data.bagId, control.dataEntry.data.slotIndex, itemLink, RIGHT, 1)
			end)
		end
	end
end

local function HookTradeHouse(eventCode, responseType, result)
	ZO_PreHook(TRADING_HOUSE.m_searchResultsList.dataTypes[1], "setupCallback", function( ... )
		local control, data = ...
		if ( control.slotControlType and control.slotControlType == 'listSlot' and control.dataEntry.data.slotIndex ) then
			local itemLink = GetTradingHouseSearchResultItemLink(control.dataEntry.data.slotIndex, LINK_STYLE_BRACKETS)
			AddTrackingIndicator(control, nil, nil, itemLink, RIGHT, 2)
		end
	end)
	ZO_PreHook(TRADING_HOUSE.m_postedItemsList.dataTypes[2], "setupCallback", function( ... )
		local control, data = ...
		if ( control.slotControlType and control.slotControlType == 'listSlot' and control.dataEntry.data.slotIndex ) then
			local itemLink = GetTradingHouseListingItemLink(control.dataEntry.data.slotIndex, LINK_STYLE_BRACKETS)
			AddTrackingIndicator(control, nil, nil, itemLink, RIGHT, 3)
		end
	end)
end

local function HookBuyback( ... )
	ZO_PreHook(ZO_BuyBackList.dataTypes[1], "setupCallback", function( ... )
		local control, data = ...
		if ( control.slotControlType and control.slotControlType == 'listSlot' and control.dataEntry.data.slotIndex ) then
			local itemLink = GetBuybackItemLink(control.dataEntry.data.slotIndex, LINK_STYLE_BRACKETS)
			AddTrackingIndicator(control, control.dataEntry.data.bagId, control.dataEntry.data.slotIndex, itemLink, RIGHT, 1)
		end
	end)
end

--------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Modify cooking station behavior and handle coloring categories and adding optional stat and writ tracking icons.
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function ClearStationSelection()
	local selectedNode = StationControl.recipeTree.selectedNode
	if selectedNode then selectedNode.selected = false selectedNode = nil end
	StationControl:RefreshRecipeList()
	StationControl.resultTooltip:SetHidden(true)
	stationSelection = ""
	for ingredientIndex, ingredientSlot in ipairs(StationControl.ingredientRows) do
		ingredientSlot:ClearItem()
	end
end

local function ToggleStationTooltips(control, option) -- Handles turning cooking station item tooltips on and off.
	if option == 1 then
		InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
		if ESOMRL.SV.cOpts.sttshow == 1 then
			SetTooltipText(InformationTooltip, L.ESOMRL_LTTHIDE)
		else
			SetTooltipText(InformationTooltip, L.ESOMRL_LTTSHOW)
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if NoRecipes() == false then
			if ESOMRL.SV.cOpts.sttshow == 1 then
				StationControl.resultTooltip:SetHidden(true)
				ESOMRL.SV.cOpts.sttshow = 0
				ClearTooltip(InformationTooltip)
				InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
				SetTooltipText(InformationTooltip, L.ESOMRL_LTTSHOW)
			else
				local selectedNode = StationControl.recipeTree.selectedNode
				if selectedNode and selectedNode.selected then
					StationControl.resultTooltip:SetHidden(false)
				end
				ESOMRL.SV.cOpts.sttshow = 1
				ClearTooltip(InformationTooltip)
				InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
				SetTooltipText(InformationTooltip, L.ESOMRL_LTTHIDE)
			end
		end
	end
end

local function HighlightTracked(control, option) -- Toggle highlighting tracked items at the cooking station.
	if option == 1 then
		InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
		if ESOMRL.SV.cOpts.stmarked == 1 then
			SetTooltipText(InformationTooltip, L.ESOMRL_STRACKN)
		else
			SetTooltipText(InformationTooltip, L.ESOMRL_STRACKY)
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if ESOMRL.SV.cOpts.stmarked == 1 then
			ESOMRL.SV.cOpts.stmarked = 0
			StationControl:RefreshRecipeList()
			ClearTooltip(InformationTooltip)
			InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_STRACKY)
		else
			ESOMRL.SV.cOpts.stmarked = 1
			StationControl:RefreshRecipeList()
			ClearTooltip(InformationTooltip)
			InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_STRACKN)
		end
	end
end

local function SetStationIcons(control, list, headerText, hasWrit, hasTrack) -- Add stat indicator if enabled.
	local recipeListName, numRecipes, upIcon, downIcon, overIcon, disabledIcon, createSound = GetRecipeListInfo(list)
	local recipeListName = string.upper(recipeListName)
	local function markTrack() control.text:SetText(control.text:GetText() .. DB.StationIcons[9]) end
	local function markWrit() control.text:SetText(control.text:GetText() .. DB.StationIcons[8]) end
	if list == 1 or list == 8 then
		if recipeListName == headerText then
			if ESOMRL.ASV.aOpts.stationstats then control.text:SetText(headerText .. DB.StationIcons[1]) else control.text:SetText(headerText) end
			if hasTrack then markTrack() end
			if hasWrit then markWrit() end
		end
	elseif list== 2 or list == 9 then
		if recipeListName == headerText then
			if ESOMRL.ASV.aOpts.stationstats then control.text:SetText(headerText .. DB.StationIcons[2]) else control.text:SetText(headerText) end
			if hasTrack then markTrack() end
			if hasWrit then markWrit() end
		end
	elseif list == 3 or list == 10 then
		if recipeListName == headerText then
			if ESOMRL.ASV.aOpts.stationstats then control.text:SetText(headerText .. DB.StationIcons[3]) else control.text:SetText(headerText) end
			if hasTrack then markTrack() end
			if hasWrit then markWrit() end
		end
	elseif list == 4 or list == 11 then
		if recipeListName == headerText then
			if ESOMRL.ASV.aOpts.stationstats then control.text:SetText(headerText .. DB.StationIcons[4]) else control.text:SetText(headerText) end
			if hasTrack then markTrack() end
			if hasWrit then markWrit() end
		end
	elseif list == 5 or list == 12 then
		if recipeListName == headerText then
			if ESOMRL.ASV.aOpts.stationstats then control.text:SetText(headerText .. DB.StationIcons[5]) else control.text:SetText(headerText) end
			if hasTrack then markTrack() end
			if hasWrit then markWrit() end
		end
	elseif list == 6 or list == 13 then
		if recipeListName == headerText then
			if ESOMRL.ASV.aOpts.stationstats then control.text:SetText(headerText .. DB.StationIcons[6]) else control.text:SetText(headerText) end
			if hasTrack then markTrack() end
			if hasWrit then markWrit() end
		end
	elseif list == 7 or list == 14 then
		if recipeListName == headerText then
			if ESOMRL.ASV.aOpts.stationstats then control.text:SetText(headerText .. DB.StationIcons[7]) else control.text:SetText(headerText) end
			if hasTrack then markTrack() end
			if hasWrit then markWrit() end
		end
	elseif list == 15 then
		if recipeListName == headerText then
			control.text:SetText(headerText)
			if hasTrack then markTrack() end
		end
	end
end

local function SetStationColors(control, opt, recipeListIndex) -- Colorize cooking station recipe headers by quality.
	local headerText = control.text:GetText():gsub("  |.*",''):gsub(" |.*",'')
	local function setWritColor(recipeListName, header, cv, hasWrit, hasTrack)
		local r, g, b
		if cv == 1 then
			if recipeListName == headerText then
				if DB.StationControls[header].c == nil then DB.StationControls[header].c = control end
				if opt == 1 then r, g, b = 0.176, 0.773, 0.055 else r, g, b = 0.314, 0.91, 0.192 end
				control.text:SetColor(r,g,b)
				SetStationIcons(control, header, headerText, hasWrit, hasTrack)
			end
		elseif cv == 2 then
			if recipeListName == headerText then
				if DB.StationControls[header].c == nil then DB.StationControls[header].c = control end
				if opt == 1 then r, g, b = 0.227, 0.573, 1 else r, g, b = 0.365, 0.71, 1 end
				control.text:SetColor(r,g,b)
				SetStationIcons(control, header, headerText, hasWrit, hasTrack)
			end
		elseif cv == 3 then
			if recipeListName == headerText then
				if DB.StationControls[header].c == nil then DB.StationControls[header].c = control end
				if opt == 1 then r, g, b = 0.627, 0.18, 0.969 else r, g, b = 0.765, 0.318, 1 end
				control.text:SetColor(r,g,b)
				SetStationIcons(control, header, headerText, hasWrit, hasTrack)
			end
		elseif cv == 4 then
			if recipeListName == headerText then
				if DB.StationControls[header].c == nil then DB.StationControls[header].c = control end
				if opt == 1 then r, g, b = 0.933, 0.792, 0.165 else r, g, b = 1, 0.929, 0.302 end
				control.text:SetColor(r,g,b)
				SetStationIcons(control, header, headerText, hasWrit, hasTrack)
			end
		end
	end
	if recipeListIndex ~= nil then
		local countWrit = false
		local countTrack = false
		local recipeListName, numRecipes, upIcon, downIcon, overIcon, disabledIcon, createSound = GetRecipeListInfo(recipeListIndex)
		recipeListName = string.upper(recipeListName)
		local cv = DB.StationChecking[recipeListIndex].cv
		for i = 1, #DB.StationChecking[recipeListIndex].tr do
			if DB.StationChecking[recipeListIndex].tr[i].writ == 1 then countWrit = true break end
		end
		for i = 1, #DB.StationChecking[recipeListIndex].tr do
			if DB.StationChecking[recipeListIndex].tr[i].track == 1 then countTrack = true break end
		end
		setWritColor(recipeListName, recipeListIndex, cv, countWrit, countTrack)
	else
		for c = 1, 16 do
			local countWrit = false
			local countTrack = false
			local recipeListName, numRecipes, upIcon, downIcon, overIcon, disabledIcon, createSound = GetRecipeListInfo(c)
			recipeListName = string.upper(recipeListName)
			local cv = DB.StationChecking[c].cv
			for i = 1, #DB.StationChecking[c].tr do
				if DB.StationChecking[c].tr[i].writ == 1 then countWrit = true break end
			end
			for i = 1, #DB.StationChecking[c].tr do
				if DB.StationChecking[c].tr[i].track == 1 then countTrack = true break end
			end
			setWritColor(recipeListName, c, cv, countWrit, countTrack)
		end
	end
end

local function SetWritColor(self)
	local recipeListIndex = self.data.recipeListIndex
	local recipeIndex = self.data.recipeIndex
	local itemLink = GetRecipeResultItemLink(recipeListIndex, recipeIndex)
	local itemId = GetItemIdFromLink(itemLink)
	if ESOMRL.ResultMaterials[itemId] == nil then return end
	local control = DB.StationControls[recipeListIndex].c
	local saved = GetRecipeSavedVar(LookupPosition(ESOMRL.ResultMaterials, itemId), 1)
	local sub = ESOMRL.ResultMaterials[itemId].sub
	DB.StationChecking[recipeListIndex].tr[sub].writ = 0
	DB.StationChecking[recipeListIndex].tr[sub].track = 0
	if itemId == DB.WritQuest.reqId1 or itemId == DB.WritQuest.reqId2 then
		DB.StationChecking[recipeListIndex].tr[sub].writ = 1
		SetStationColors(control, 1, recipeListIndex)
		if not self.enabled then
			return 0.878,0.878,0.298
		elseif self.mouseover then
			return 1,1,0.394
		else
			return 1,1,0
		end
	elseif ( ( (saved == 1) or (saved == 2) ) and ( ESOMRL.SV.cOpts.stmarked == 1 ) ) then
		DB.StationChecking[recipeListIndex].tr[sub].track = 1
		SetStationColors(control, 1, recipeListIndex)
		if not self.enabled then
			return 0,0.82,0.91
		elseif self.selected or self.mouseover then
			return 0.2,1,1
		else
			return 0,0.91,1
		end
	elseif not self.enabled then
		SetStationColors(control, 1, recipeListIndex)
		return GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_DISABLED)
	elseif self.selected then
		SetStationColors(control, 1, recipeListIndex)
		return GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_SELECTED)
	elseif self.mouseover then
		SetStationColors(control, 1, recipeListIndex)
		return GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_HIGHLIGHT)
	elseif self.meetsLevelReq and self.meetsQualityReq then
		SetStationColors(control, 1, recipeListIndex)
		return GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_NORMAL)
	end
	return ZO_ERROR_COLOR:UnpackRGBA()
end

local function UpdateStationTooltip()
	local resultTooltip = StationControl.resultTooltip
	if stnorecs == 0 then
		if ESOMRL.SV.cOpts.sttshow == 0 then resultTooltip:SetHidden(true) else resultTooltip:SetHidden(false) end
	else
		resultTooltip:SetHidden(true)
	end
	if itemId == DB.WritQuest.reqId1 or itemId == DB.WritQuest.reqId2 then
		resultTooltip:AddVerticalPadding(20)
		resultTooltip:AddLine('|cffff00' .. L.ESOMRL_NWRIT .. '|r', 'ZoFontHeader2', 1, 1, 1, CENTER, MODIFY_TEXT_TYPE_NONE, CENTER, false)
	end
end
	
local function CraftStoreHook()
	local ESOMRL_ZO_ProvisionerStartInteract = ZO_Provisioner.StartInteract
	ZO_Provisioner.StartInteract = function(self,...)
		StationControl = self -- Store the station control for interactive use elsewhere.
		ESOMRL_ZO_ProvisionerStartInteract(self,...)
	end
end

local function CheckCraftStore()
	if CraftStoreFixed_Cook and CraftStore_Account.Default[GetDisplayName()]["$AccountWide"][GetWorldName()]["option"][7] ~= nil and CraftStore_Account.Default[GetDisplayName()]["$AccountWide"][GetWorldName()]["option"][7] == true then return true else return false end
end

local function HookStation()
	if (CheckCraftStore()) then
		CraftStoreHook()
	--	return -- No need to modify the cooking station when using CraftStore
	end

	-- These hooks handle coloring tracked/writ recipes and recipe categories by quality
	local ESOMRL_ZO_IconHeader_Setup = ZO_IconHeader_Setup
	ZO_IconHeader_Setup = function(control,...)
		ESOMRL_ZO_IconHeader_Setup(control,...)
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) then SetStationColors(control, 1) end
	end
	local ESOMRL_ZO_IconHeader_OnMouseEnter = ZO_IconHeader_OnMouseEnter
	ZO_IconHeader_OnMouseEnter = function(control)
		ESOMRL_ZO_IconHeader_OnMouseEnter(control)
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) then SetStationColors(control, 2) end
	end
	local ESOMRL_ZO_IconHeader_OnMouseExit = ZO_IconHeader_OnMouseExit
	ZO_IconHeader_OnMouseExit = function(control)
		ESOMRL_ZO_IconHeader_OnMouseExit(control)
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) then SetStationColors(control, 1) end
	end
	local ESOMRL_ZO_ProvisionerRow_GetTextColor = ZO_ProvisionerRow_GetTextColor
	ZO_ProvisionerRow_GetTextColor = function(self)
		local descriptor = ZO_ProvisionerTopLevelTabs["m_object"]:GetSelectedDescriptor()
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) and descriptor < 3 then
			return SetWritColor(self)
	--	elseif (not CheckCraftStore()) then
		else
			if not self.enabled then
				return GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_DISABLED)
			elseif self.selected then
				return GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_SELECTED)
			elseif self.mouseover then
				return GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_HIGHLIGHT)
			elseif self.meetsLevelReq and self.meetsQualityReq then
				return GetInterfaceColor(INTERFACE_COLOR_TYPE_TEXT_COLORS, INTERFACE_TEXT_COLOR_NORMAL)
			end
			return ZO_ERROR_COLOR:UnpackRGBA()
		end
	end

	-- Runs when the crafting station UI has been loaded 
	local ESOMRL_ZO_ProvisionerStartInteract = ZO_Provisioner.StartInteract
	ZO_Provisioner.StartInteract = function(self,...)
		StationControl = self -- Store the station control for interactive use elsewhere.
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) then -- only if cooking...
			if ESOMRL.ASV.aOpts.noFilters == true then -- Auto-uncheck filters on cooking station startup.
				self.savedVars.haveIngredientsChecked = false
				self.savedVars.haveSkillsChecked = false
				ZO_CheckButton_SetCheckState(self.haveIngredientsCheckBox, false)
				ZO_CheckButton_SetCheckState(self.haveSkillsCheckBox, false)
			end
			StationControl:RefreshRecipeList()
		end
		ESOMRL_ZO_ProvisionerStartInteract(self,...)
	end

	-- Close open categories and clear selection when changing tabs
	local ESOMRL_ZO_ProvisionerOnTabFilterChanged = ZO_Provisioner.OnTabFilterChanged
	ZO_Provisioner.OnTabFilterChanged = function(self, filterData)
		ESOMRL_ZO_ProvisionerOnTabFilterChanged(self, filterData)
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) then
			statNodes = {}
			ClearStationSelection()
		end
	end

	-- Prevent auto-opening any recipe categories when station loads
	ZO_PreHook("ZO_IconHeader_Setup", function(self,...)
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) then
			local treeNode = self.node.tree
			if treeNode.exclusive or treeNode.open then
				treeNode.exclusive = false
				treeNode.open = false
			end
		end
	end)

	-- Runs when a recipe category is clicked
	ZO_PreHook("ZO_TreeHeader_OnMouseUp", function(self,...)
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) then
			local treeNode = {tnode = self}
			local data = self.node.data
			local name = LowerSpaceSpecial(data.name)
			if self.node.open == false then -- Keep track of open categories so we can manually close them
				--d(name)
				if statNodes[name] == nil then
					statNodes[name] = treeNode
				end
			else
				statNodes[name] = nil
			end
		end
	end)

	-- Runs when a recipe name is clicked
	ZO_PreHook("ZO_TreeEntry_OnMouseUp", function(self, upInside)
		local data = self.node.data
		local recipeListIndex = data.recipeListIndex
		local recipeIndex = data.recipeIndex
		local itemLink = GetRecipeResultItemLink(recipeListIndex, recipeIndex)
		local itemId = GetItemIdFromLink(itemLink)
		local name, icon = GetRecipeResultItemInfo(recipeListIndex, recipeIndex)
		stationSelection = LanguageFormat(name) -- Store the selected recipe name for searching MRL.
		local descriptor = ZO_ProvisionerTopLevelTabs["m_object"]:GetSelectedDescriptor()
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING and (not CheckCraftStore()) and descriptor < 3 then
			zo_callLater(UpdateStationTooltip, 200)
		end
	end)

end

local function OnEndCraftingStationInteract(eventCode)
	local function CleanExit()
		ESOMRL:SetHidden(true)
		StationControl = nil
		EVENT_MANAGER:UnregisterForEvent(ESOMRL.Name, EVENT_END_CRAFTING_STATION_INTERACT)
	end
	if (CheckCraftStore()) then
		CleanExit()
	else
		statNodes = {}
		ClearStationSelection()
		CleanExit()
	end
end

local function OnCraftingStationInteract(eventCode, craftSkill)
	if craftSkill == CRAFTING_TYPE_PROVISIONING then
		if NoRecipes() then stnorecs = 1 else stnorecs = 0 end
		local controlName = ZO_ProvisionerTopLevelNavigationContainer
		if ESOMRL:IsHidden() then
			ESOMRLStationTitle:SetHidden(true)
			ESOMRLResetStationNavigation:SetHidden(false)
			ESOMRLFindInMRLButton:SetHidden(false)
			ESOMRLStationTooltipButton:SetHidden(false)
			ESOMRLStationTrackingButton:SetHidden(false)
			if (CheckCraftStore()) then -- Station navigation offset for CraftStore
				local csts = CraftStoreFixed_CookTopSection
				CraftStoreFixed_Cook:SetHeight(841) -- 801
				csts:ClearAnchors() csts:SetAnchor(TOPLEFT, ESOMRL, BOTTOMLEFT, 0, 40)
				ESOMRL:ClearAnchors() ESOMRL:SetAnchor(TOPLEFT, CraftStoreFixed_Cook, TOPLEFT, 10, 10) ESOMRL:SetHidden(false)
				ESOMRLStationTitle:SetHidden(false)
				ESOMRLResetStationNavigation:SetHidden(true)
				ESOMRLFindInMRLButton:SetHidden(true)
				ESOMRLStationTooltipButton:SetHidden(true)
				ESOMRLStationTrackingButton:SetHidden(true)
			elseif KhrillMasterCook_settings then -- Station navigation offset for Krill Master Cook
				ESOMRL:ClearAnchors() ESOMRL:SetAnchor(TOPLEFT, controlName, TOPLEFT, 58, -89) ESOMRL:SetHidden(false)
			else -- Standard station navigation position
				ESOMRL:ClearAnchors() ESOMRL:SetAnchor(TOPLEFT, controlName, TOPLEFT, 0, -86) ESOMRL:SetHidden(false)
			end
		end
		EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_END_CRAFTING_STATION_INTERACT, OnEndCraftingStationInteract)
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Initialize the scrollable list of recipes and handle various category and search functions.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function SetListItem(control,data) -- Hook the ScrollList data table.
	local listitemtext = control:GetNamedChild( 'Name' )
	listitemtext:SetText( data.RecipeName )
end

local function TextSearch(refresh) -- Populate recipe list based on text search.
	local datalist = ZO_ScrollList_GetDataList(ESOMRL_MainFrameListFrameList)
	local searchtext
	local known
	local row = 0
	if editText ~= 2 then
		openTab = 50
		if editText == 1 or refresh == 1 then
			ZO_ScrollList_Clear(ESOMRL_MainFrameListFrameList) 
			if refresh == 1 then searchtext = LowerSpaceSpecial(textInput) else searchtext = LowerSpaceSpecial(ESOMRL_MainFrameListFrameSearchBox:GetText()) end
			if searchtext == "" then
				for k,v in pairs(datalist) do datalist[k] = nil end
				ZO_ScrollList_Commit(ESOMRL_MainFrameListFrameList, datalist)
			else
				for k,v in pairs(searchTable) do searchTable[k] = nil end
				if refresh ~= 1 then searchtext = searchtext:gsub("%W",''):lower() end

				if setMode == 1 then
					for i = 1, pRecipes do
						local name = LowerSpaceSpecial(GetItemLinkName(ESOMRL.ProvisioningLinks[i].item))
						local tid = tonumber(ESOMRL.ProvisioningLinks[i].id)
						local color = tostring(ESOMRL.RecipeMaterials[tid].color)
						local flevel = ESOMRL.ProvisioningLabels[i].level
	
						if (string.find(flevel,"Any Level") ~= nil) then
							flevel = "alspfdesomrl"
						else
							flevel = flevel:gsub("%(Level",''):gsub("Food%)",''):gsub("Drink%)",''):gsub("%s",'')
						end

						local function CheckRange(st, fl)
							local sText = zo_strlower(LanguageFormat(st))
							local fLower = fl:gsub("%-.*",''):gsub("%D",'')
							local fUpper = fl:gsub(".*%-",''):gsub("%D",'')
	
							if flevel == "alspfdesomrl" then
								if sText == "cp1" then
									return true
								else
									return false
								end
							elseif (string.find(sText,"cp%d") ~= nil) then
								if string.find(fl,"CP") ~= nil then
									sText = sText:gsub("%D",'')
									if tonumber(sText) ~= nil and tonumber(fLower) ~= nil and tonumber(fUpper) ~= nil then
										if ((tonumber(sText) >= tonumber(fLower)) and (tonumber(sText) <= tonumber(fUpper))) then
											return true
										else
											return false
										end
									else
										return false
									end
								else
									return false
								end
							elseif (string.find(sText,"%d") ~= nil) then
								if string.find(fl,"CP") == nil then
									sText = sText:gsub("%D",'')
									if tonumber(sText) ~= nil and tonumber(fLower) ~= nil and tonumber(fUpper) ~= nil then
										if (tonumber(sText) >= tonumber(fLower)) and (tonumber(sText) <= tonumber(fUpper)) then
											return true
										else
											return false
										end
									else
										return false
									end
								else
									return false
								end
							else
								return false
							end
						end
	
						if (string.find(name,searchtext) ~= nil) or CheckRange(searchtext, flevel) then
							local ln = tonumber(ESOMRL.ProvisioningLabels[i].position)
							local level = FormatTooltipText(ESOMRL.ProvisioningLabels[ln].level) 
							local saved = GetRecipeSavedVar(ln, 0)
							local writSet = 0
							if IsItemLinkRecipeKnown(ESOMRL.ProvisioningLinks[ln].link) == false then
								local resLink = ESOMRL.ProvisioningLinks[ln].item
								if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
									writSet = 1
									if saved == 1 then known = DB.IconStrings[6] elseif saved == 0 then known = DB.IconStrings[5] end
								else
									writSet = (GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2) and 1 or 0
									if saved == 1 then known = DB.IconStrings[4] elseif saved == 0 then known = DB.IconStrings[3] end
								end
							else
								if saved == 2 then known = DB.IconStrings[2] elseif saved == 3 then known = DB.IconStrings[1] end
							end
							row = row + 1
							local linkname = GetItemLinkName(ESOMRL.ProvisioningLinks[ln].item)
							local namestring = LanguageFormat(linkname)
							searchTable[row] = { pos = ln, writ = writSet, text = known .. color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r', name = linkname }
							datalist[row] = ZO_ScrollList_CreateDataEntry( 1, 
							{
								RecipeName = known .. color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r',
							}
							)
						end
					end
				else
					for i = 1, fRecipes do
						local name = LowerSpaceSpecial(GetItemLinkName(ESOMRL.FurnitureLinks[i].item))
						local tid = tonumber(ESOMRL.FurnitureLinks[i].id)
						local color = tostring(ESOMRL.RecipeFMaterials[tid].color)

						if string.find(name,searchtext) ~= nil then
							local ln = tonumber(ESOMRL.RecipeFMaterials[tid].position)
							local saved = GetRecipeSavedVar(ln, 100)
							if IsItemLinkRecipeKnown(ESOMRL.FurnitureLinks[ln].link) == false then
								if saved == 1 then known = DB.IconStrings[4] elseif saved == 0 then known = DB.IconStrings[3] end
							else
								if saved == 2 then known = DB.IconStrings[2] elseif saved == 3 then known = DB.IconStrings[1] end
							end
							row = row + 1
							local linkname = GetItemLinkName(ESOMRL.FurnitureLinks[ln].item)
							local namestring = LanguageFormat(linkname)
							local rType = tostring(ESOMRL.RecipeFTypes[ESOMRL.RecipeFMaterials[tid].rType])
							searchTable[row] = { pos = ln, text = known .. color .. namestring .. '|r' .. rType, name = linkname, link = ESOMRL.FurnitureLinks[ln].item }
							datalist[row] = ZO_ScrollList_CreateDataEntry( 1, 
							{
								RecipeName = known .. color .. namestring .. '|r' .. rType,
							}
							)
						end
					end
				end

				ZO_ScrollList_Commit(ESOMRL_MainFrameListFrameList, datalist)
			end
		end
	end
end

local function IngRecipeSearch() -- Show list of recipes containing selected ingredients.
	local datalist = ZO_ScrollList_GetDataList(ESOMRL_MainFrameListFrameList)
	for k,v in pairs(datalist) do datalist[k] = nil end
	local ingredients = {}
	local recipes = {}
	local match = {}
	local known
	local color
	local row = 0
	ESOMRL_MainFrameSelectionFrame:SetHidden(false)
	ESOMRL_MainFrameListFrame:SetHidden(false)
	ESOMRL_MainFrameIngredientsFrame:SetHidden(true)
	setPage = 1
	textInput = ""
	openTab = 52
	for i = 1, 50 do
		local savedvar = GetIngredientSavedVar(i)
		if savedvar == 1 then
			local namelookup = LowerSpaceSpecial(GetItemLinkName(ESOMRL.IngredientLinks[i].link))
			if not Contains(ingredients, namelookup) then
				table.insert(ingredients, #ingredients + 1, namelookup)
			end
		end
	end
	if #ingredients == 0 then
		ZO_ScrollList_Commit(ESOMRL_MainFrameListFrameList, datalist)
	else
		for k,v in pairs(searchTable) do searchTable[k] = nil end
		for r = 1, pRecipes do
			local tempdb = {}
			local tempcount = 0
			local linkcheck = ESOMRL.ProvisioningLinks[r].link
			local numIngredients = GetItemLinkRecipeNumIngredients(linkcheck)
			for n = 1, numIngredients do
				local ingredientName,_ = GetItemLinkRecipeIngredientInfo(linkcheck, n)
				ingredientName = LowerSpaceSpecial(ingredientName)
				table.insert(tempdb, #tempdb + 1, ingredientName)
			end
			for c = 1, #ingredients do
				if not Contains(tempdb, ingredients[c]) then
					tempcount = tempcount - 1
				end
			end
			if tempcount >= 0 then match[r] = 1 else match[r] = 0 end
		end
		for m = 1, pRecipes do
			if match[m] == 1 then
				local ln = tonumber(ESOMRL.ProvisioningLabels[m].position)
				if ln > 0 and ln < 127 then
					color = '|c00ff00'
				elseif ln > 126 and ln < 235 then
					color = '|c3a92ff'
				elseif ln > 234 and ln < 266 then
					color = '|ca02ef7'
				elseif ln > 265 and ln < 392 then
					color = '|c00ff00'
				elseif ln > 391 and ln < 500 then
					color = '|c3a92ff'
				elseif ln > 499 and ln < 531 then
					color = '|ca02ef7'
				elseif ln >= 531 then
					color = '|ceeca2a'
				end
				local level = FormatTooltipText(ESOMRL.ProvisioningLabels[ln].level) 
				local saved = GetRecipeSavedVar(ln, 0)
				if IsItemLinkRecipeKnown(ESOMRL.ProvisioningLinks[ln].link) == false then
					local resLink = ESOMRL.ProvisioningLinks[ln].item
					if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
						if saved == 1 then known = DB.IconStrings[6] elseif saved == 0 then known = DB.IconStrings[5] end
					else
						if saved == 1 then known = DB.IconStrings[4] elseif saved == 0 then known = DB.IconStrings[3] end
					end
				else
					if saved == 2 then known = DB.IconStrings[2] elseif saved == 3 then known = DB.IconStrings[1] end
				end
				row = row + 1
				local linkname = GetItemLinkName(ESOMRL.ProvisioningLinks[ln].item)
				local namestring = LanguageFormat(linkname)
				searchTable[row] = { pos = ln, text = known .. color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r', name = linkname }
				datalist[row] = ZO_ScrollList_CreateDataEntry( 1, 
				{
					RecipeName = known .. color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r',
				}
				)
			end
		end
		ZO_ScrollList_Commit(ESOMRL_MainFrameListFrameList, datalist)
	end
end

local function StationSearch(control, option) -- Search MRL for the selected cooking station item.
	if option == 1 then
		InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_SEARCHS)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		local control = GetControl('ESOMRL_MainFrame')
		if control:IsHidden() then
			ESOMRL.SetLanguageFormat()
			control:SetHidden(false)
			RestorePosition()
			openTab = 1
			SetTrackedRecipeCountText()
			RecipePanelRestore()
		end
		textInput = stationSelection
		ESOMRL_MainFrameSelectionFrame:SetHidden(false)
		ESOMRL_MainFrameFurnitureFrame:SetHidden(true)
		ESOMRL_MainFrameListFrame:SetHidden(false)
		ESOMRL_MainFrameIngredientsFrame:SetHidden(true)
		setPage = 1
		setMode = 1
		TextSearch(1)
		if stationSelection ~= "" then
			zo_callLater(UpdateStationTooltip, 200)
		end
	end
end

local function ResetStationNavigation(control, option) -- Closes all open recipe trees at the cooking station.
	if option == 1 then
		InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_RESETNAV)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		for k, v in pairs(statNodes) do
			local tnode = v.tnode
			local treeNode = tnode.node.tree
			--d(tnode.node.data.name)
			if tnode.node.open then
				ZO_TreeHeader_OnMouseUp(tnode, true)
			end
			treeNode.exclusive = false
			treeNode.open = false
		end
		statNodes = {}
		ClearStationSelection()
	end
end

local function GetWrits() -- Populate recipe list based on current active writ requirements.
	if GetWritID() ~= 0 then
		local datalist = ZO_ScrollList_GetDataList(ESOMRL_MainFrameListFrameList)
		local searchtext
		local known
		local color
		local row = 0
		openTab = 51
		textInput = ''
		ZO_ScrollList_Clear(ESOMRL_MainFrameListFrameList)
		for k,v in pairs(datalist) do datalist[k] = nil end
		for k,v in pairs(searchTable) do searchTable[k] = nil end
		for i = 1, pRecipes do
			local lookup = GetItemIdFromLink(ESOMRL.ProvisioningLinks[i].item)
			if lookup == DB.WritQuest.reqId1 or lookup == DB.WritQuest.reqId2 then
				local ln = tonumber(ESOMRL.ProvisioningLabels[i].position)
				if ln > 0 and ln < 127 then
					color = '|c00ff00'
				elseif ln > 126 and ln < 235 then
					color = '|c3a92ff'
				elseif ln > 234 and ln < 266 then
					color = '|ca02ef7'
				elseif ln > 265 and ln < 392 then
					color = '|c00ff00'
				elseif ln > 391 and ln < 500 then
					color = '|c3a92ff'
				elseif ln > 499 and ln < 531 then
					color = '|ca02ef7'
				elseif ln >= 531 then
					color = '|ceeca2a'
				end
				local level = FormatTooltipText(ESOMRL.ProvisioningLabels[ln].level) 
				local saved = GetRecipeSavedVar(ln, 0)
				if IsItemLinkRecipeKnown(ESOMRL.ProvisioningLinks[ln].link) == false then
					if saved == 1 then known = DB.IconStrings[6] elseif saved == 0 then known = DB.IconStrings[5] end
				else
					if saved == 2 then known = DB.IconStrings[2] elseif saved == 3 then known = DB.IconStrings[1] end
				end
				row = row + 1
				local linkname = GetItemLinkName(ESOMRL.ProvisioningLinks[ln].item)
				local namestring = LanguageFormat(linkname)
				searchTable[row] = { pos = ln, writ = 1, text = known .. color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r', name = linkname }
				datalist[row] = ZO_ScrollList_CreateDataEntry( 1, 
				{
					RecipeName = known .. color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r',
				}
				)
			end
		end
		ZO_ScrollList_Commit(ESOMRL_MainFrameListFrameList, datalist)
		editText = 2
	end
end

local function TrackNavigationTier(list) -- Marks the entire tier as tracked if holding shift when clicking the button.
	local lower
	local upper
	local furnitureDB
	if list == 1 then
		lower = 1
		upper = 42
	elseif list == 2 then
		lower = 43
		upper = 84
	elseif list == 3 then
		lower = 85
		upper = 126
	elseif list == 4 then
		lower = 127
		upper = 162
	elseif list == 5 then
		lower = 163
		upper = 198
	elseif list == 6 then
		lower = 199
		upper = 234
	elseif list == 7 then
		lower = 235
		upper = 265
	elseif list == 8 then
		lower = 266
		upper = 307
	elseif list == 9 then
		lower = 308
		upper = 349
	elseif list == 10 then
		lower = 350
		upper = 391
	elseif list == 11 then
		lower = 392
		upper = 427
	elseif list == 12 then
		lower = 428
		upper = 463
	elseif list == 13 then
		lower = 464
		upper = 499
	elseif list == 14 then
		lower = 500
		upper = 530
	elseif list == 15 then
		lower = 531
		upper = 535
	elseif list == 16 then
		lower = 536
		upper = 539
	elseif list == 17 then
		lower = 540
		upper = 548
	elseif list == 18 then
		lower = 549
		upper = 557
	elseif list == 19 then
		lower = 558
		upper = 561
	elseif list == 20 then
		lower = 562
		upper = 564
	elseif list > 100 then
		if list == 101 then
			furnitureDB = ESOMRL.FurnitureLinksAlchemy1
			upper = (#ESOMRL.FurnitureLinksAlchemy1 > 0) and #ESOMRL.FurnitureLinksAlchemy1 or 0
		elseif list == 102 then
			furnitureDB = ESOMRL.FurnitureLinksAlchemy2
			upper = (#ESOMRL.FurnitureLinksAlchemy2 > 0) and #ESOMRL.FurnitureLinksAlchemy2 or 0
		elseif list == 103 then
			furnitureDB = ESOMRL.FurnitureLinksAlchemy3
			upper = (#ESOMRL.FurnitureLinksAlchemy3 > 0) and #ESOMRL.FurnitureLinksAlchemy3 or 0
		elseif list == 104 then
			furnitureDB = ESOMRL.FurnitureLinksAlchemy4
			upper = (#ESOMRL.FurnitureLinksAlchemy4 > 0) and #ESOMRL.FurnitureLinksAlchemy4 or 0
		elseif list == 105 then
			furnitureDB = ESOMRL.FurnitureLinksAlchemy5
			upper = (#ESOMRL.FurnitureLinksAlchemy5 > 0) and #ESOMRL.FurnitureLinksAlchemy5 or 0
		elseif list == 106 then
			furnitureDB = ESOMRL.FurnitureLinksBlacksmithing1
			upper = (#ESOMRL.FurnitureLinksBlacksmithing1 > 0) and #ESOMRL.FurnitureLinksBlacksmithing1 or 0
		elseif list == 107 then
			furnitureDB = ESOMRL.FurnitureLinksBlacksmithing2
			upper = (#ESOMRL.FurnitureLinksBlacksmithing2 > 0) and #ESOMRL.FurnitureLinksBlacksmithing2 or 0
		elseif list == 108 then
			furnitureDB = ESOMRL.FurnitureLinksBlacksmithing3
			upper = (#ESOMRL.FurnitureLinksBlacksmithing3 > 0) and #ESOMRL.FurnitureLinksBlacksmithing3 or 0
		elseif list == 109 then
			furnitureDB = ESOMRL.FurnitureLinksBlacksmithing4
			upper = (#ESOMRL.FurnitureLinksBlacksmithing4 > 0) and #ESOMRL.FurnitureLinksBlacksmithing4 or 0
		elseif list == 110 then
			furnitureDB = ESOMRL.FurnitureLinksBlacksmithing5
			upper = (#ESOMRL.FurnitureLinksBlacksmithing5 > 0) and #ESOMRL.FurnitureLinksBlacksmithing5 or 0
		elseif list == 111 then
			furnitureDB = ESOMRL.FurnitureLinksClothing1
			upper = (#ESOMRL.FurnitureLinksClothing1 > 0) and #ESOMRL.FurnitureLinksClothing1 or 0
		elseif list == 112 then
			furnitureDB = ESOMRL.FurnitureLinksClothing2
			upper = (#ESOMRL.FurnitureLinksClothing2 > 0) and #ESOMRL.FurnitureLinksClothing2 or 0
		elseif list == 113 then
			furnitureDB = ESOMRL.FurnitureLinksClothing3
			upper = (#ESOMRL.FurnitureLinksClothing3 > 0) and #ESOMRL.FurnitureLinksClothing3 or 0
		elseif list == 114 then
			furnitureDB = ESOMRL.FurnitureLinksClothing4
			upper = (#ESOMRL.FurnitureLinksClothing4 > 0) and #ESOMRL.FurnitureLinksClothing4 or 0
		elseif list == 115 then
			furnitureDB = ESOMRL.FurnitureLinksClothing5
			upper = (#ESOMRL.FurnitureLinksClothing5 > 0) and #ESOMRL.FurnitureLinksClothing5 or 0
		elseif list == 116 then
			furnitureDB = ESOMRL.FurnitureLinksEnchanting1
			upper = (#ESOMRL.FurnitureLinksEnchanting1 > 0) and #ESOMRL.FurnitureLinksEnchanting1 or 0
		elseif list == 117 then
			furnitureDB = ESOMRL.FurnitureLinksEnchanting2
			upper = (#ESOMRL.FurnitureLinksEnchanting2 > 0) and #ESOMRL.FurnitureLinksEnchanting2 or 0
		elseif list == 118 then
			furnitureDB = ESOMRL.FurnitureLinksEnchanting3
			upper = (#ESOMRL.FurnitureLinksEnchanting3 > 0) and #ESOMRL.FurnitureLinksEnchanting3 or 0
		elseif list == 119 then
			furnitureDB = ESOMRL.FurnitureLinksEnchanting4
			upper = (#ESOMRL.FurnitureLinksEnchanting4 > 0) and #ESOMRL.FurnitureLinksEnchanting4 or 0
		elseif list == 120 then
			furnitureDB = ESOMRL.FurnitureLinksEnchanting5
			upper = (#ESOMRL.FurnitureLinksEnchanting5 > 0) and #ESOMRL.FurnitureLinksEnchanting5 or 0
		elseif list == 121 then
			furnitureDB = ESOMRL.FurnitureLinksProvisioning1
			upper = (#ESOMRL.FurnitureLinksProvisioning1 > 0) and #ESOMRL.FurnitureLinksProvisioning1 or 0
		elseif list == 122 then
			furnitureDB = ESOMRL.FurnitureLinksProvisioning2
			upper = (#ESOMRL.FurnitureLinksProvisioning2 > 0) and #ESOMRL.FurnitureLinksProvisioning2 or 0
		elseif list == 123 then
			furnitureDB = ESOMRL.FurnitureLinksProvisioning3
			upper = (#ESOMRL.FurnitureLinksProvisioning3 > 0) and #ESOMRL.FurnitureLinksProvisioning3 or 0
		elseif list == 124 then
			furnitureDB = ESOMRL.FurnitureLinksProvisioning4
			upper = (#ESOMRL.FurnitureLinksProvisioning4 > 0) and #ESOMRL.FurnitureLinksProvisioning4 or 0
		elseif list == 125 then
			furnitureDB = ESOMRL.FurnitureLinksProvisioning5
			upper = (#ESOMRL.FurnitureLinksProvisioning5 > 0) and #ESOMRL.FurnitureLinksProvisioning5 or 0
		elseif list == 126 then
			furnitureDB = ESOMRL.FurnitureLinksWoodworking1
			upper = (#ESOMRL.FurnitureLinksWoodworking1 > 0) and #ESOMRL.FurnitureLinksWoodworking1 or 0
		elseif list == 127 then
			furnitureDB = ESOMRL.FurnitureLinksWoodworking2
			upper = (#ESOMRL.FurnitureLinksWoodworking2 > 0) and #ESOMRL.FurnitureLinksWoodworking2 or 0
		elseif list == 128 then
			furnitureDB = ESOMRL.FurnitureLinksWoodworking3
			upper = (#ESOMRL.FurnitureLinksWoodworking3 > 0) and #ESOMRL.FurnitureLinksWoodworking3 or 0
		elseif list == 129 then
			furnitureDB = ESOMRL.FurnitureLinksWoodworking4
			upper = (#ESOMRL.FurnitureLinksWoodworking4 > 0) and #ESOMRL.FurnitureLinksWoodworking4 or 0
		elseif list == 130 then
			furnitureDB = ESOMRL.FurnitureLinksWoodworking5
			upper = (#ESOMRL.FurnitureLinksWoodworking5 > 0) and #ESOMRL.FurnitureLinksWoodworking5 or 0
		elseif openTab == 131 then
			furnitureDB = ESOMRL.FurnitureLinksJewelry1
			upper = (#ESOMRL.FurnitureLinksJewelry1 > 0) and #ESOMRL.FurnitureLinksJewelry1 or 0
		elseif openTab == 132 then
			furnitureDB = ESOMRL.FurnitureLinksJewelry2
			upper = (#ESOMRL.FurnitureLinksJewelry2 > 0) and #ESOMRL.FurnitureLinksJewelry2 or 0
		elseif openTab == 133 then
			furnitureDB = ESOMRL.FurnitureLinksJewelry3
			upper = (#ESOMRL.FurnitureLinksJewelry3 > 0) and #ESOMRL.FurnitureLinksJewelry3 or 0
		elseif openTab == 134 then
			furnitureDB = ESOMRL.FurnitureLinksJewelry4
			upper = (#ESOMRL.FurnitureLinksJewelry4 > 0) and #ESOMRL.FurnitureLinksJewelry4 or 0
		elseif openTab == 135 then
			furnitureDB = ESOMRL.FurnitureLinksJewelry5
			upper = (#ESOMRL.FurnitureLinksJewelry5 > 0) and #ESOMRL.FurnitureLinksJewelry5 or 0
		end
	end

	if setMode == 1 then
		for i = lower, upper do
			local savedvar = GetRecipeSavedVar(i, 1)
			if savedvar == 3 then
				SetRecipeSavedVar(i, 1, 2)
				ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
			elseif savedvar == 0 then
				SetRecipeSavedVar(i, 1, 1)
				ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
			end
		end
	else
		if upper > 0 then
			for i = 1, upper do
				local pos = tonumber(furnitureDB[i].position)
				local savedvar = GetRecipeSavedVar(pos, 100)
				if savedvar == 3 then
					SetRecipeSavedVar(pos, 100, 2)
					ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
				elseif savedvar == 0 then
					SetRecipeSavedVar(pos, 100, 1)
					ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
				end
			end
		end
	end
	SetTrackedRecipeCountText()
	RefreshViews()
end

local function NavigateScrollList(list) -- Populate recipe list based on navigation item clicked.
	ZO_ScrollList_Clear(ESOMRL_MainFrameListFrameList)
	local datalist = ZO_ScrollList_GetDataList(ESOMRL_MainFrameListFrameList)
	for k,v in pairs(datalist) do datalist[k] = nil end
	for k,v in pairs(searchTable) do searchTable[k] = nil end	
	local known
	local row = 0
	local linktable
	textInput = ''
	if list == 1 then
		linktable = ESOMRL.ProvisioningLinksFood1
		openTab = 1
	elseif list == 2 then
		linktable = ESOMRL.ProvisioningLinksFood2
		openTab = 2
	elseif list == 3 then
		linktable = ESOMRL.ProvisioningLinksFood3
		openTab = 3
	elseif list == 4 then
		linktable = ESOMRL.ProvisioningLinksFood4
		openTab = 4
	elseif list == 5 then
		linktable = ESOMRL.ProvisioningLinksFood5
		openTab = 5
	elseif list == 6 then
		linktable = ESOMRL.ProvisioningLinksFood6
		openTab = 6
	elseif list == 7 then
		linktable = ESOMRL.ProvisioningLinksFood7
		openTab = 7
	elseif list == 8 then
		linktable = ESOMRL.ProvisioningLinksDrinks1
		openTab = 8
	elseif list == 9 then
		linktable = ESOMRL.ProvisioningLinksDrinks2
		openTab = 9
	elseif list == 10 then
		linktable = ESOMRL.ProvisioningLinksDrinks3
		openTab = 10
	elseif list == 11 then
		linktable = ESOMRL.ProvisioningLinksDrinks4
		openTab = 11
	elseif list == 12 then
		linktable = ESOMRL.ProvisioningLinksDrinks5
		openTab = 12
	elseif list == 13 then
		linktable = ESOMRL.ProvisioningLinksDrinks6
		openTab = 13
	elseif list == 14 then
		linktable = ESOMRL.ProvisioningLinksDrinks7
		openTab = 14
	elseif list == 15 then
		linktable = ESOMRL.ProvisioningLinksPsijic
		openTab = 15
	elseif list == 16 then
		linktable = ESOMRL.ProvisioningLinksOrsinium
		openTab = 16
	elseif list == 17 then
		linktable = ESOMRL.ProvisioningLinksWitchesFestival
		openTab = 17
	elseif list == 18 then
		linktable = ESOMRL.ProvisioningLinksNewLifeFestival
		openTab = 18
	elseif list == 19 then
		linktable = ESOMRL.ProvisioningLinksJesterFestival
		openTab = 19
	elseif list == 20 then
		linktable = ESOMRL.ProvisioningLinksClockworkCity
		openTab = 20
	elseif list > 100 then
		openTab = list
		if list == 101 then
			linktable = ESOMRL.FurnitureLinksAlchemy1
		elseif list == 102 then
			linktable = ESOMRL.FurnitureLinksAlchemy2
		elseif list == 103 then
			linktable = ESOMRL.FurnitureLinksAlchemy3
		elseif list == 104 then
			linktable = ESOMRL.FurnitureLinksAlchemy4
		elseif list == 105 then
			linktable = ESOMRL.FurnitureLinksAlchemy5
		elseif list == 106 then
			linktable = ESOMRL.FurnitureLinksBlacksmithing1
		elseif list == 107 then
			linktable = ESOMRL.FurnitureLinksBlacksmithing2
		elseif list == 108 then
			linktable = ESOMRL.FurnitureLinksBlacksmithing3
		elseif list == 109 then
			linktable = ESOMRL.FurnitureLinksBlacksmithing4
		elseif list == 110 then
			linktable = ESOMRL.FurnitureLinksBlacksmithing5
		elseif list == 111 then
			linktable = ESOMRL.FurnitureLinksClothing1
		elseif list == 112 then
			linktable = ESOMRL.FurnitureLinksClothing2
		elseif list == 113 then
			linktable = ESOMRL.FurnitureLinksClothing3
		elseif list == 114 then
			linktable = ESOMRL.FurnitureLinksClothing4
		elseif list == 115 then
			linktable = ESOMRL.FurnitureLinksClothing5
		elseif list == 116 then
			linktable = ESOMRL.FurnitureLinksEnchanting1
		elseif list == 117 then
			linktable = ESOMRL.FurnitureLinksEnchanting2
		elseif list == 118 then
			linktable = ESOMRL.FurnitureLinksEnchanting3
		elseif list == 119 then
			linktable = ESOMRL.FurnitureLinksEnchanting4
		elseif list == 120 then
			linktable = ESOMRL.FurnitureLinksEnchanting5
		elseif list == 121 then
			linktable = ESOMRL.FurnitureLinksProvisioning1
		elseif list == 122 then
			linktable = ESOMRL.FurnitureLinksProvisioning2
		elseif list == 123 then
			linktable = ESOMRL.FurnitureLinksProvisioning3
		elseif list == 124 then
			linktable = ESOMRL.FurnitureLinksProvisioning4
		elseif list == 125 then
			linktable = ESOMRL.FurnitureLinksProvisioning5
		elseif list == 126 then
			linktable = ESOMRL.FurnitureLinksWoodworking1
		elseif list == 127 then
			linktable = ESOMRL.FurnitureLinksWoodworking2
		elseif list == 128 then
			linktable = ESOMRL.FurnitureLinksWoodworking3
		elseif list == 129 then
			linktable = ESOMRL.FurnitureLinksWoodworking4
		elseif list == 130 then
			linktable = ESOMRL.FurnitureLinksWoodworking5
		elseif list == 131 then
			linktable = ESOMRL.FurnitureLinksJewelry1
		elseif list == 132 then
			linktable = ESOMRL.FurnitureLinksJewelry2
		elseif list == 133 then
			linktable = ESOMRL.FurnitureLinksJewelry3
		elseif list == 134 then
			linktable = ESOMRL.FurnitureLinksJewelry4
		elseif list == 135 then
			linktable = ESOMRL.FurnitureLinksJewelry5
		end
	end

	if setMode == 1 then
		for i = 1, #linktable do
			local ln = tonumber(linktable[i].position)
			local level = FormatTooltipText(linktable[i].level) 
			local saved = GetRecipeSavedVar(i, list)
			local writSet = 0
			if IsItemLinkRecipeKnown(linktable[i].link) == false then
				local resLink = linktable[i].item
				if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
					writSet = 1
					if saved == 1 then known = DB.IconStrings[6] elseif saved == 0 then known = DB.IconStrings[5] end
				else
					writSet = (GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2) and 1 or 0
					if saved == 1 then known = DB.IconStrings[4] elseif saved == 0 then known = DB.IconStrings[3] end
				end
			else
				if saved == 2 then known = DB.IconStrings[2] elseif saved == 3 then known = DB.IconStrings[1] end
			end
			row = row + 1
			local linkname = GetItemLinkName(linktable[i].item)
			local namestring = LanguageFormat(linkname)
			searchTable[row] = { pos = ln, writ = writSet, text = known .. linktable[i].color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r', name = linkname }
			datalist[i] = ZO_ScrollList_CreateDataEntry( 1, 
			{
				RecipeName = known .. linktable[i].color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r',
			}
			)
			ZO_ScrollList_Commit(ESOMRL_MainFrameListFrameList, datalist)
		end
	else
		if #linktable > 0 then
			for i = 1, #linktable do
				local ln = tonumber(linktable[i].position)
				local saved = GetRecipeSavedVar(ln, 100)
				if IsItemLinkRecipeKnown(linktable[i].link) == false then
					if saved == 1 then known = DB.IconStrings[4] elseif saved == 0 then known = DB.IconStrings[3] end
				else
					if saved == 2 then known = DB.IconStrings[2] elseif saved == 3 then known = DB.IconStrings[1] end
				end
				row = row + 1
				local linkname = GetItemLinkName(linktable[i].item)
				local namestring = LanguageFormat(linkname)
				local catstring = GetFurnitureSubCat(list)
				searchTable[row] = { pos = ln, text = known .. linktable[i].color .. namestring .. '|r' .. catstring, name = linkname }
				datalist[i] = ZO_ScrollList_CreateDataEntry( 1, 
				{
					RecipeName = known .. linktable[i].color .. namestring .. '|r' .. catstring,
				}
				)
			end
		else
			searchTable[1] = { pos = 1, text = ' ', name = ' ' }
			datalist[1] = ZO_ScrollList_CreateDataEntry( 1, 
			{
				RecipeName = L.ESOMRL_NOENTRIES,
			}
			)
		end
		ZO_ScrollList_Commit(ESOMRL_MainFrameListFrameList, datalist)
	end
end

local function TrackQuality(tier) -- Mark all recipes of a selected improvement rank as tracked when shift-clicking.
	local dtable = ESOMRL.ProvisioningLinks
	for i = 1, pRecipes do
		local rank = GetItemLinkRecipeRankRequirement(dtable[i].link)
		if rank == tier then
			local savedvar = GetRecipeSavedVar(i, 200)
			if savedvar == 3 then
				SetRecipeSavedVar(i, 200, 2)
				ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
			elseif savedvar == 0 then
				SetRecipeSavedVar(i, 200, 1)
				ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
			end
		end
	end
	SetTrackedRecipeCountText()
	RefreshViews()
end

local function NavigateQuality(tier) -- Populate recipe list based on improvement rank clicked.
	ZO_ScrollList_Clear(ESOMRL_MainFrameListFrameList)
	local datalist = ZO_ScrollList_GetDataList(ESOMRL_MainFrameListFrameList)
	for k,v in pairs(datalist) do datalist[k] = nil end
	for k,v in pairs(searchTable) do searchTable[k] = nil end
	local linktable = ESOMRL.ProvisioningLabels
	local dtable = ESOMRL.ProvisioningLinks
	local ctable = ESOMRL.RecipeMaterials
	openTab = 53
	local known
	local row = 0
	for i = 1, pRecipes do
		local rank = GetItemLinkRecipeRankRequirement(dtable[i].link)
		if rank == tier then
			local ln = tonumber(linktable[i].position)
			local level = FormatTooltipText(linktable[i].level) 
			local saved = GetRecipeSavedVar(i, 200)
			local cindex = tonumber(dtable[i].id)
			local writSet = 0
			if IsItemLinkRecipeKnown(dtable[i].link) == false then
				local resLink = dtable[i].item
				if GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2 then
					writSet = 1
					if saved == 1 then known = DB.IconStrings[6] elseif saved == 0 then known = DB.IconStrings[5] end
				else
					writSet = (GetItemIdFromLink(resLink) == DB.WritQuest.reqId1 or GetItemIdFromLink(resLink) == DB.WritQuest.reqId2) and 1 or 0
					if saved == 1 then known = DB.IconStrings[4] elseif saved == 0 then known = DB.IconStrings[3] end
				end
			else
				if saved == 2 then known = DB.IconStrings[2] elseif saved == 3 then known = DB.IconStrings[1] end
			end
			row = row + 1
			local linkname = GetItemLinkName(dtable[i].item)
			local namestring = LanguageFormat(linkname)
			searchTable[row] = { pos = ln, writ = writSet, text = known .. ctable[cindex].color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r', name = linkname }
			datalist[row] = ZO_ScrollList_CreateDataEntry( 1, 
			{
				RecipeName = known .. ctable[cindex].color .. namestring .. '|r' .. '|cffffff ' .. level .. '|r',
			}
			)
			ZO_ScrollList_Commit(ESOMRL_MainFrameListFrameList, datalist)
		end
	end
end

local function TrackShown() -- Sets all recipes currently displayed in the result list to tracked.
	if checkingShown == 1 then
		return
	else -- button spam prevention as this function can take a while.
		checkingShown = 1
		if #searchTable > 0 then
			local datalist = ZO_ScrollList_GetDataList(ESOMRL_MainFrameListFrameList)
			if setMode == 1 then
				for s = 1, #searchTable do
					local linkname = searchTable[s].name
					local name = searchTable[s].text
					for i = 1, pRecipes do
						local namestring = GetItemLinkName(ESOMRL.ProvisioningLinks[i].item)
						if linkname == namestring then
							local known = name:gsub(DB.IconStrings[1],''):gsub(DB.IconStrings[2],''):gsub(DB.IconStrings[3],''):gsub(DB.IconStrings[4],'')
							local savedvar = GetRecipeSavedVar(i, 200)
							if savedvar == 3 then
								SetRecipeSavedVar(i, 200, 2)
								known = DB.IconStrings[2] .. known
								searchTable[s].text = known
								ESOMRL_MainFrameListFrameList.data[s].data.RecipeName = known
								ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
							elseif savedvar == 0 then
								SetRecipeSavedVar(i, 200, 1)
								known = DB.IconStrings[4] .. known
								searchTable[s].text = known
								ESOMRL_MainFrameListFrameList.data[s].data.RecipeName = known
								ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
							end
						end
					end
				end
			else
				for s = 1, #searchTable do
					local linkname = searchTable[s].name
					local name = searchTable[s].text
					for i = 1, fRecipes do
						local namestring = GetItemLinkName(ESOMRL.FurnitureLinks[i].item)
						if linkname == namestring then
							local known = name:gsub(DB.IconStrings[1],''):gsub(DB.IconStrings[2],''):gsub(DB.IconStrings[3],''):gsub(DB.IconStrings[4],'')
							local savedvar = GetRecipeSavedVar(i, 100)
							if savedvar == 3 then
								SetRecipeSavedVar(i, 100, 2)
								known = DB.IconStrings[2] .. known
								searchTable[s].text = known
								ESOMRL_MainFrameListFrameList.data[s].data.RecipeName = known
								ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
							elseif savedvar == 0 then
								SetRecipeSavedVar(i, 100, 1)
								known = DB.IconStrings[4] .. known
								searchTable[s].text = known
								ESOMRL_MainFrameListFrameList.data[s].data.RecipeName = known
								ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
							end
						end
					end
				end
			end
			SetTrackedRecipeCountText()
			ZO_ScrollList_RefreshVisible(ESOMRL_MainFrameListFrameList)
			RefreshViews()
		end
		checkingShown = 0
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Handle the clicking of items in the active recipe list, setting and clearing tracked items.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function CountRecipeTrackingList(option, tier) -- Counts the current number of recipes you are tracking.
	if setMode == 1 then
		if option == 0 then
			ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist - 1
		elseif option == 1 then
			ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
		end
	else
		if option == 0 then
			ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist - 1
		elseif option == 1 then
			ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
		end
	end
	SetTrackedRecipeCountText()
	ZO_ScrollList_RefreshVisible(ESOMRL_MainFrameListFrameList)
	RefreshViews()
end

local function TrackAllRecipes(option) -- Set all recipes to tracked.
	if option == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameCloseButton, TOPLEFT, 8, -44, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_TRACKALL)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if setMode == 1 then
			for i = 1, pRecipes do
				local savedvar = GetRecipeSavedVar(i, 1)
				if savedvar == 3 then
					SetRecipeSavedVar(i, 1, 2)
					ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
				elseif savedvar == 0 then
					SetRecipeSavedVar(i, 1, 1)
					ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
				end
			end
		else
			for i = 1, fRecipes do
				local savedvar = GetRecipeSavedVar(i, 100)
				if savedvar == 3 then
					SetRecipeSavedVar(i, 100, 2)
					ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
				elseif savedvar == 0 then
					SetRecipeSavedVar(i, 100, 1)
					ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
				end
			end
		end
		if (openTab ~= 50) and (openTab ~= 51) and (openTab ~= 52) and (openTab ~= 53) then
			NavigateScrollList(openTab)
		else
			openTab = 1
			NavigateScrollList(openTab)
			textInput = ''
		end
		ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
		ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
		SetTrackedRecipeCountText()
		RecipePanelRestore()
		RefreshViews()
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING then
			StationControl:RefreshRecipeList() -- Refreshes category marks
		end
	end
end

local function TrackUnknownRecipes(option) -- Track recipes the current selected character does not know.
	if option == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameCloseButton, TOPLEFT, 8, -44, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_TRACKUNKNOWN)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if setMode == 1 then
			for p = 1, pRecipes do
				local savedvar = GetRecipeSavedVar(p, 1)
				if not Contains(ESOMRL.ASV.pRecipeKnown[p].names, currentChar) then
					if savedvar == 3 then
						SetRecipeSavedVar(p, 1, 2)
						ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
					elseif savedvar == 0 then
						SetRecipeSavedVar(p, 1, 1)
						ESOMRL.SV.cOpts.recipetracklist = ESOMRL.SV.cOpts.recipetracklist + 1
					end
				end
			end
		else
			for f = 1, fRecipes do
				local savedvar = GetRecipeSavedVar(f, 100)
				if not Contains(ESOMRL.ASV.fRecipeKnown[f].names, currentChar) then
					if savedvar == 3 then
						SetRecipeSavedVar(f, 100, 2)
						ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
					elseif savedvar == 0 then
						SetRecipeSavedVar(f, 100, 1)
						ESOMRL.SV.cOpts.furnituretracklist = ESOMRL.SV.cOpts.furnituretracklist + 1
					end
				end
			end
		end
		if (openTab ~= 50) and (openTab ~= 51) and (openTab ~= 52) and (openTab ~= 53) then
			NavigateScrollList(openTab)
		else
			openTab = 1
			NavigateScrollList(openTab)
			textInput = ''
		end
		ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
		ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
		SetTrackedRecipeCountText()
		RecipePanelRestore()
		RefreshViews()
		if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING then
			StationControl:RefreshRecipeList() -- Refreshes category marks
		end
	end
end

local function ClearTrackedRecipes() -- Clears all tracked recipes, removing tooltip line.
	if setMode == 1 then
		ESOMRL.SV.cOpts.recipetracklist = 0
		for i = 1, pRecipes do
			if IsItemLinkRecipeKnown(ESOMRL.ProvisioningLinks[i].link) == false then
				SetRecipeSavedVar(i, 1, 0)
			else
				if GetRecipeSavedVar(i, 1) == 2 then
					SetRecipeSavedVar(i, 1, 3)
				end
			end
		end
	else
		ESOMRL.SV.cOpts.furnituretracklist = 0
		for i = 1, fRecipes do
			if IsItemLinkRecipeKnown(ESOMRL.FurnitureLinks[i].link) == false then
				SetRecipeSavedVar(i, 100, 0)
			else
				if GetRecipeSavedVar(i, 100) == 2 then
					SetRecipeSavedVar(i, 100, 3)
				end
			end
		end
	end
	if (openTab ~= 50) and (openTab ~= 51) and (openTab ~= 52) and (openTab ~= 53) then
		NavigateScrollList(openTab)
	elseif setMode == 0 then
		NavigateScrollList(102)
	else
		openTab = 1
		NavigateScrollList(openTab)
		textInput = ''
	end
	ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
	ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
	SetTrackedRecipeCountText()
	RecipePanelRestore()
	RefreshViews()
	if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING then
		StationControl:RefreshRecipeList() -- Refreshes category marks
	end
end

local function RecipeListClick(clicktext, button) -- Handles clicking and shift-clicking of list items.
	-- 0 = unknown
	-- 1 = unknown tracked
	-- 2 = known tracked
	-- 3 = known
	if openTab < 50 or openTab > 100 then
		for i = 1, #searchTable do
			local name = searchTable[i].text
			local writ = searchTable[i].writ
			local known = name:gsub(DB.IconStrings[1],''):gsub(DB.IconStrings[2],''):gsub(DB.IconStrings[3],''):gsub(DB.IconStrings[4],''):gsub(DB.IconStrings[5],''):gsub(DB.IconStrings[6],'')
			if clicktext == name then
				if IsShiftKeyDown() == false then
					local savedval
					if setMode == 1 then
						savedval = GetRecipeSavedVar(i, openTab)
						local iconstring = ''
						if savedval == 0 then
							SetRecipeSavedVar(i, openTab, 1)
							iconstring = (writ == 1) and DB.IconStrings[6] or DB.IconStrings[4]
							known = iconstring .. known
							searchTable[i].text = known
							ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
							CountRecipeTrackingList(1, openTab)
						elseif savedval == 1 then
							SetRecipeSavedVar(i, openTab, 0)
							iconstring = (writ == 1) and DB.IconStrings[5] or DB.IconStrings[3]
							known = iconstring .. known
							searchTable[i].text = known
							ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
							CountRecipeTrackingList(0, openTab)
						elseif savedval == 2 then
							SetRecipeSavedVar(i, openTab, 3)
							known = DB.IconStrings[1] .. known
							searchTable[i].text = known
							ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
							CountRecipeTrackingList(0, openTab)
						elseif savedval == 3 then
							SetRecipeSavedVar(i, openTab, 2)
							known = DB.IconStrings[2] .. known
							searchTable[i].text = known
							ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
							CountRecipeTrackingList(1, openTab)
						end
						ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
						ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
						RecipePanelRestore()
					else
						if button == 2 then -- preview furniture with a Crown version (API limitation) on right-click
							local fLink
							if openTab == 101 then
								fLink = ESOMRL.FurnitureLinksAlchemy1[i].link
							elseif openTab == 102 then
								fLink = ESOMRL.FurnitureLinksAlchemy2[i].link
							elseif openTab == 103 then
								fLink = ESOMRL.FurnitureLinksAlchemy3[i].link
							elseif openTab == 104 then
								fLink = ESOMRL.FurnitureLinksAlchemy4[i].link
							elseif openTab == 105 then
								fLink = ESOMRL.FurnitureLinksAlchemy5[i].link
							elseif openTab == 106 then
								fLink = ESOMRL.FurnitureLinksBlacksmithing1[i].link
							elseif openTab == 107 then
								fLink = ESOMRL.FurnitureLinksBlacksmithing2[i].link
							elseif openTab == 108 then
								fLink = ESOMRL.FurnitureLinksBlacksmithing3[i].link
							elseif openTab == 109 then
								fLink = ESOMRL.FurnitureLinksBlacksmithing4[i].link
							elseif openTab == 110 then
								fLink = ESOMRL.FurnitureLinksBlacksmithing5[i].link
							elseif openTab == 111 then
								fLink = ESOMRL.FurnitureLinksClothing1[i].link
							elseif openTab == 112 then
								fLink = ESOMRL.FurnitureLinksClothing2[i].link
							elseif openTab == 113 then
								fLink = ESOMRL.FurnitureLinksClothing3[i].link
							elseif openTab == 114 then
								fLink = ESOMRL.FurnitureLinksClothing4[i].link
							elseif openTab == 115 then
								fLink = ESOMRL.FurnitureLinksClothing5[i].link
							elseif openTab == 116 then
								fLink = ESOMRL.FurnitureLinksEnchanting1[i].link
							elseif openTab == 117 then
								fLink = ESOMRL.FurnitureLinksEnchanting2[i].link
							elseif openTab == 118 then
								fLink = ESOMRL.FurnitureLinksEnchanting3[i].link
							elseif openTab == 119 then
								fLink = ESOMRL.FurnitureLinksEnchanting4[i].link
							elseif openTab == 120 then
								fLink = ESOMRL.FurnitureLinksEnchanting5[i].link
							elseif openTab == 121 then
								fLink = ESOMRL.FurnitureLinksProvisioning1[i].link
							elseif openTab == 122 then
								fLink = ESOMRL.FurnitureLinksProvisioning2[i].link
							elseif openTab == 123 then
								fLink = ESOMRL.FurnitureLinksProvisioning3[i].link
							elseif openTab == 124 then
								fLink = ESOMRL.FurnitureLinksProvisioning4[i].link
							elseif openTab == 125 then
								fLink = ESOMRL.FurnitureLinksProvisioning5[i].link
							elseif openTab == 126 then
								fLink = ESOMRL.FurnitureLinksWoodworking1[i].link
							elseif openTab == 127 then
								fLink = ESOMRL.FurnitureLinksWoodworking2[i].link
							elseif openTab == 128 then
								fLink = ESOMRL.FurnitureLinksWoodworking3[i].link
							elseif openTab == 129 then
								fLink = ESOMRL.FurnitureLinksWoodworking4[i].link
							elseif openTab == 130 then
								fLink = ESOMRL.FurnitureLinksWoodworking5[i].link
							elseif openTab == 131 then
								fLink = ESOMRL.FurnitureLinksJewelry1[i].link
							elseif openTab == 132 then
								fLink = ESOMRL.FurnitureLinksJewelry2[i].link
							elseif openTab == 133 then
								fLink = ESOMRL.FurnitureLinksJewelry3[i].link
							elseif openTab == 134 then
								fLink = ESOMRL.FurnitureLinksJewelry4[i].link
							elseif openTab == 135 then
								fLink = ESOMRL.FurnitureLinksJewelry5[i].link
							end

							if PREVIEW:CanPreviewItemLink(fLink) then
								PREVIEW:PreviewItemLink(fLink)
							else
								if ESOMRL.SV.cOpts.debugmode == true then
									d("API - Can only preview furniture that has Crown version.")
								end
							end
						else
							local varPos
							if openTab == 101 then
								varPos = tonumber(ESOMRL.FurnitureLinksAlchemy1[i].position)
							elseif openTab == 102 then
								varPos = tonumber(ESOMRL.FurnitureLinksAlchemy2[i].position)
							elseif openTab == 103 then
								varPos = tonumber(ESOMRL.FurnitureLinksAlchemy3[i].position)
							elseif openTab == 104 then
								varPos = tonumber(ESOMRL.FurnitureLinksAlchemy4[i].position)
							elseif openTab == 105 then
								varPos = tonumber(ESOMRL.FurnitureLinksAlchemy5[i].position)
							elseif openTab == 106 then
								varPos = tonumber(ESOMRL.FurnitureLinksBlacksmithing1[i].position)
							elseif openTab == 107 then
								varPos = tonumber(ESOMRL.FurnitureLinksBlacksmithing2[i].position)
							elseif openTab == 108 then
								varPos = tonumber(ESOMRL.FurnitureLinksBlacksmithing3[i].position)
							elseif openTab == 109 then
								varPos = tonumber(ESOMRL.FurnitureLinksBlacksmithing4[i].position)
							elseif openTab == 110 then
								varPos = tonumber(ESOMRL.FurnitureLinksBlacksmithing5[i].position)
							elseif openTab == 111 then
								varPos = tonumber(ESOMRL.FurnitureLinksClothing1[i].position)
							elseif openTab == 112 then
								varPos = tonumber(ESOMRL.FurnitureLinksClothing2[i].position)
							elseif openTab == 113 then
								varPos = tonumber(ESOMRL.FurnitureLinksClothing3[i].position)
							elseif openTab == 114 then
								varPos = tonumber(ESOMRL.FurnitureLinksClothing4[i].position)
							elseif openTab == 115 then
								varPos = tonumber(ESOMRL.FurnitureLinksClothing5[i].position)
							elseif openTab == 116 then
								varPos = tonumber(ESOMRL.FurnitureLinksEnchanting1[i].position)
							elseif openTab == 117 then
								varPos = tonumber(ESOMRL.FurnitureLinksEnchanting2[i].position)
							elseif openTab == 118 then
								varPos = tonumber(ESOMRL.FurnitureLinksEnchanting3[i].position)
							elseif openTab == 119 then
								varPos = tonumber(ESOMRL.FurnitureLinksEnchanting4[i].position)
							elseif openTab == 120 then
								varPos = tonumber(ESOMRL.FurnitureLinksEnchanting5[i].position)
							elseif openTab == 121 then
								varPos = tonumber(ESOMRL.FurnitureLinksProvisioning1[i].position)
							elseif openTab == 122 then
								varPos = tonumber(ESOMRL.FurnitureLinksProvisioning2[i].position)
							elseif openTab == 123 then
								varPos = tonumber(ESOMRL.FurnitureLinksProvisioning3[i].position)
							elseif openTab == 124 then
								varPos = tonumber(ESOMRL.FurnitureLinksProvisioning4[i].position)
							elseif openTab == 125 then
								varPos = tonumber(ESOMRL.FurnitureLinksProvisioning5[i].position)
							elseif openTab == 126 then
								varPos = tonumber(ESOMRL.FurnitureLinksWoodworking1[i].position)
							elseif openTab == 127 then
								varPos = tonumber(ESOMRL.FurnitureLinksWoodworking2[i].position)
							elseif openTab == 128 then
								varPos = tonumber(ESOMRL.FurnitureLinksWoodworking3[i].position)
							elseif openTab == 129 then
								varPos = tonumber(ESOMRL.FurnitureLinksWoodworking4[i].position)
							elseif openTab == 130 then
								varPos = tonumber(ESOMRL.FurnitureLinksWoodworking5[i].position)
							elseif openTab == 131 then
								varPos = tonumber(ESOMRL.FurnitureLinksJewelry1[i].position)
							elseif openTab == 132 then
								varPos = tonumber(ESOMRL.FurnitureLinksJewelry2[i].position)
							elseif openTab == 133 then
								varPos = tonumber(ESOMRL.FurnitureLinksJewelry3[i].position)
							elseif openTab == 134 then
								varPos = tonumber(ESOMRL.FurnitureLinksJewelry4[i].position)
							elseif openTab == 135 then
								varPos = tonumber(ESOMRL.FurnitureLinksJewelry5[i].position)
							end

							savedval = GetRecipeSavedVar(varPos, 100)
							if savedval == 0 then
								SetRecipeSavedVar(varPos, 100, 1)
								known = DB.IconStrings[4] .. known
								searchTable[i].text = known
								ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
								CountRecipeTrackingList(1, openTab)
							elseif savedval == 1 then
								SetRecipeSavedVar(varPos, 100, 0)
								known = DB.IconStrings[3] .. known
								searchTable[i].text = known
								ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
								CountRecipeTrackingList(0, openTab)
							elseif savedval == 2 then
								SetRecipeSavedVar(varPos, 100, 3)
								known = DB.IconStrings[1] .. known
								searchTable[i].text = known
								ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
								CountRecipeTrackingList(0, openTab)
							elseif savedval == 3 then
								SetRecipeSavedVar(varPos, 100, 2)
								known = DB.IconStrings[2] .. known
								searchTable[i].text = known
								ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
								CountRecipeTrackingList(1, openTab)
							end
							ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
							ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
							RecipePanelRestore()
						end
					end
				elseif IsShiftKeyDown() == true then
					local index = 0
					if openTab == 1 then
						index = i
					elseif openTab == 2 then
						index = i + 42
					elseif openTab == 3 then
						index = i + 84
					elseif openTab == 4 then
						index = i + 126
					elseif openTab == 5 then
						index = i + 162
					elseif openTab == 6 then
						index = i + 198
					elseif openTab == 7 then
						index = i + 234
					elseif openTab == 8 then
						index = i + 265
					elseif openTab == 9 then
						index = i + 307
					elseif openTab == 10 then
						index = i + 349
					elseif openTab == 11 then
						index = i + 391
					elseif openTab == 12 then
						index = i + 427
					elseif openTab == 13 then
						index = i + 463
					elseif openTab == 14 then
						index = i + 499
					elseif openTab == 15 then
						index = i + 530
					elseif openTab == 16 then
						index = i + 535
					elseif openTab == 17 then
						index = i + 539
					elseif openTab == 18 then
						index = i + 548
					elseif openTab == 19 then
						index = i + 557
					elseif openTab == 20 then
						index = i + 561
					elseif openTab > 100 then
						local linktable
						if openTab == 101 then
							linktable = ESOMRL.FurnitureLinksAlchemy1
						elseif openTab == 102 then
							linktable = ESOMRL.FurnitureLinksAlchemy2
						elseif openTab == 103 then
							linktable = ESOMRL.FurnitureLinksAlchemy3
						elseif openTab == 104 then
							linktable = ESOMRL.FurnitureLinksAlchemy4
						elseif openTab == 105 then
							linktable = ESOMRL.FurnitureLinksAlchemy5
						elseif openTab == 106 then
							linktable = ESOMRL.FurnitureLinksBlacksmithing1
						elseif openTab == 107 then
							linktable = ESOMRL.FurnitureLinksBlacksmithing2
						elseif openTab == 108 then
							linktable = ESOMRL.FurnitureLinksBlacksmithing3
						elseif openTab == 109 then
							linktable = ESOMRL.FurnitureLinksBlacksmithing4
						elseif openTab == 110 then
							linktable = ESOMRL.FurnitureLinksBlacksmithing5
						elseif openTab == 111 then
							linktable = ESOMRL.FurnitureLinksClothing1
						elseif openTab == 112 then
							linktable = ESOMRL.FurnitureLinksClothing2
						elseif openTab == 113 then
							linktable = ESOMRL.FurnitureLinksClothing3
						elseif openTab == 114 then
							linktable = ESOMRL.FurnitureLinksClothing4
						elseif openTab == 115 then
							linktable = ESOMRL.FurnitureLinksClothing5
						elseif openTab == 116 then
							linktable = ESOMRL.FurnitureLinksEnchanting1
						elseif openTab == 117 then
							linktable = ESOMRL.FurnitureLinksEnchanting2
						elseif openTab == 118 then
							linktable = ESOMRL.FurnitureLinksEnchanting3
						elseif openTab == 119 then
							linktable = ESOMRL.FurnitureLinksEnchanting4
						elseif openTab == 120 then
							linktable = ESOMRL.FurnitureLinksEnchanting5
						elseif openTab == 121 then
							linktable = ESOMRL.FurnitureLinksProvisioning1
						elseif openTab == 122 then
							linktable = ESOMRL.FurnitureLinksProvisioning2
						elseif openTab == 123 then
							linktable = ESOMRL.FurnitureLinksProvisioning3
						elseif openTab == 124 then
							linktable = ESOMRL.FurnitureLinksProvisioning4
						elseif openTab == 125 then
							linktable = ESOMRL.FurnitureLinksProvisioning5
						elseif openTab == 126 then
							linktable = ESOMRL.FurnitureLinksWoodworking1
						elseif openTab == 127 then
							linktable = ESOMRL.FurnitureLinksWoodworking2
						elseif openTab == 128 then
							linktable = ESOMRL.FurnitureLinksWoodworking3
						elseif openTab == 129 then
							linktable = ESOMRL.FurnitureLinksWoodworking4
						elseif openTab == 130 then
							linktable = ESOMRL.FurnitureLinksWoodworking5
						elseif openTab == 131 then
							linktable = ESOMRL.FurnitureLinksJewelry1
						elseif openTab == 132 then
							linktable = ESOMRL.FurnitureLinksJewelry2
						elseif openTab == 133 then
							linktable = ESOMRL.FurnitureLinksJewelry3
						elseif openTab == 134 then
							linktable = ESOMRL.FurnitureLinksJewelry4
						elseif openTab == 135 then
							linktable = ESOMRL.FurnitureLinksJewelry5
						end
						index = tonumber(linktable[i].position)
					end

					local ChatEditControl = CHAT_SYSTEM.textEntry.editControl
					if not ChatEditControl:HasFocus() then 
						StartChatInput()
					end
					
					if setMode == 1 then
						if ESOMRL.SV.cOpts.tooltipstyle == 0 then
							ChatEditControl:InsertText(ESOMRL.ProvisioningLinks[index].link)
						elseif ESOMRL.SV.cOpts.tooltipstyle == 1 then
							ChatEditControl:InsertText(ESOMRL.ProvisioningLinks[index].item)
						end
					else
						if ESOMRL.SV.cOpts.tooltipstyle == 0 then
							ChatEditControl:InsertText(ESOMRL.FurnitureLinks[index].link)
						elseif ESOMRL.SV.cOpts.tooltipstyle == 1 then
							ChatEditControl:InsertText(ESOMRL.FurnitureLinks[index].item)
						end
					end
				end
			end
		end
	else
		for i = 1, #searchTable do
			local row = searchTable[i].pos
			local name = searchTable[i].text
			local writ = searchTable[i].writ
			local known = name:gsub(DB.IconStrings[1],''):gsub(DB.IconStrings[2],''):gsub(DB.IconStrings[3],''):gsub(DB.IconStrings[4],''):gsub(DB.IconStrings[5],''):gsub(DB.IconStrings[6],'')
			if clicktext == name then
				if setMode == 1 then
					if IsShiftKeyDown() == false then
						local savedval = GetRecipeSavedVar(row, openTab)
						local iconstring = ''
						if savedval == 0 then
							SetRecipeSavedVar(row, openTab, 1)
							iconstring = (writ == 1) and DB.IconStrings[6] or DB.IconStrings[4]
							known = iconstring .. known
							searchTable[i].text = known
							ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
							CountRecipeTrackingList(1, openTab)
						elseif savedval == 1 then
							SetRecipeSavedVar(row, openTab, 0)
							iconstring = (writ == 1) and DB.IconStrings[5] or DB.IconStrings[3]
							known = iconstring .. known
							searchTable[i].text = known
							ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
							CountRecipeTrackingList(0, openTab)
						elseif savedval == 2 then
							SetRecipeSavedVar(row, openTab, 3)
							known = DB.IconStrings[1] .. known
							searchTable[i].text = known
							ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
							CountRecipeTrackingList(0, openTab)
						elseif savedval == 3 then
							SetRecipeSavedVar(row, openTab, 2)
							known = DB.IconStrings[2] .. known
							searchTable[i].text = known
							ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
							CountRecipeTrackingList(1, openTab)
						end
						ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
						ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
						RecipePanelRestore()
					elseif IsShiftKeyDown() == true then
						local ChatEditControl = CHAT_SYSTEM.textEntry.editControl
						if not ChatEditControl:HasFocus() then 
							StartChatInput() 
						end
						if ESOMRL.SV.cOpts.tooltipstyle == 0 then
							ChatEditControl:InsertText(ESOMRL.ProvisioningLinks[row].link)
						elseif ESOMRL.SV.cOpts.tooltipstyle == 1 then
							ChatEditControl:InsertText(ESOMRL.ProvisioningLinks[row].item)
						end
					end
				else
					if IsShiftKeyDown() == false then
						if button == 2 then -- preview furniture with a Crown version (API limitation) on right-click
							local fLink = searchTable[i].link

							if PREVIEW:CanPreviewItemLink(fLink) then
								PREVIEW:PreviewItemLink(fLink)
							else
								if ESOMRL.SV.cOpts.debugmode == true then
									d("API - Can only preview furniture that has Crown version.")
								end
							end
						else
							local savedval = GetRecipeSavedVar(row, 100)
							if savedval == 0 then
								SetRecipeSavedVar(row, 100, 1)
								known = DB.IconStrings[4] .. known
								searchTable[i].text = known
								ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
								CountRecipeTrackingList(1, openTab)
							elseif savedval == 1 then
								SetRecipeSavedVar(row, 100, 0)
								known = DB.IconStrings[3] .. known
								searchTable[i].text = known
								ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
								CountRecipeTrackingList(0, openTab)
							elseif savedval == 2 then
								SetRecipeSavedVar(row, 100, 3)
								known = DB.IconStrings[1] .. known
								searchTable[i].text = known
								ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
								CountRecipeTrackingList(0, openTab)
							elseif savedval == 3 then
								SetRecipeSavedVar(row, 100, 2)
								known = DB.IconStrings[2] .. known
								searchTable[i].text = known
								ESOMRL_MainFrameListFrameList.data[i].data.RecipeName = known
								CountRecipeTrackingList(1, openTab)
							end
							ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
							ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
							RecipePanelRestore()
						end
					elseif IsShiftKeyDown() == true then
						local ChatEditControl = CHAT_SYSTEM.textEntry.editControl
						if not ChatEditControl:HasFocus() then 
							StartChatInput() 
						end
						if ESOMRL.SV.cOpts.tooltipstyle == 0 then
							ChatEditControl:InsertText(ESOMRL.FurnitureLinks[row].link)
						elseif ESOMRL.SV.cOpts.tooltipstyle == 1 then
							ChatEditControl:InsertText(ESOMRL.FurnitureLinks[row].item)
						end
					end
				end
			end
		end
	end
	if GetCraftingInteractionType() == CRAFTING_TYPE_PROVISIONING then
		StationControl:RefreshRecipeList() -- Refreshes category marks
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Handle the clicking of items in the ingredients panel.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function CountIngredientTrackingList(option) -- Counts the current number of ingredients you are tracking.
	if option == 0 then
		ESOMRL.SV.cOpts.ingtracklist = ESOMRL.SV.cOpts.ingtracklist - 1
		SetTrackedIngredientCountText()
	elseif option == 1 then
		ESOMRL.SV.cOpts.ingtracklist = ESOMRL.SV.cOpts.ingtracklist + 1
		SetTrackedIngredientCountText()
	end
	RefreshViews()
end

local function ClearTrackedIngredients() -- Clears all tracked ingredients, removing tooltip line.
	ESOMRL.SV.cOpts.ingtracklist = 0
	for i = 1, 50 do
		SetIngredientSavedVar(i, 0)
		SetTrackedIngredientCountText()
	end
	ESOMRL.SV.cOpts.junkunmarkedingredients = 0
	ESOMRL.SV.cOpts.destroyunmarkedingredients = 0
	IngredientPanelSetup()
	RefreshViews()
end

local function SelectAllIngredients(control, option) -- Button function to mark all ingredients.
	if option == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -43, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_TRACKALLING)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		for i = 1, 50 do
			if GetIngredientSavedVar(i) == 0 then
				SetIngredientSavedVar(i, 1)
				CountIngredientTrackingList(1)
			end
		end
		ESOMRL.SV.cOpts.junkunmarkedingredients = 0
		ESOMRL.SV.cOpts.destroyunmarkedingredients = 0
		SetTrackedIngredientCountText()
		IngredientPanelSetup()
		RefreshViews()
	end
end

local function FindIngredientRecipes(option) -- Find recipes containing selected ingredients.
	if option == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -65, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_FINDRECIPES)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		IngRecipeSearch()
	end
end

local function IngredientListClick(control, option, index) -- Handles clicking and shift-clicking of ingredients.
	if option == 1 then
		if ESOMRL.SV.cOpts.lttshow == 1 then
			local IngredientItemTooltip = IngredientItemTooltipControl
			local itemLink = ESOMRL.IngredientLinks[index].link
			InitializeTooltip(IngredientItemTooltip, ESOMRL_MainFrame, TOPLEFT, 10, -8, TOPRIGHT)
			PopupTooltip.SetLink(IngredientItemTooltip, itemLink)
		end
		if GetIngredientSavedVar(index) == 1 then
			control:SetColor(1,1,0,1)
			control:SetAlpha(.5)
		elseif GetIngredientSavedVar(index) == 0 then
			control:SetColor(1,1,1,1)
			control:SetAlpha(1)
		end
	elseif option == 2 then
		local IngredientItemTooltip = IngredientItemTooltipControl
		ClearTooltip(IngredientItemTooltip)
		if GetIngredientSavedVar(index) == 1 then
			control:SetColor(1,1,0,1)
			control:SetAlpha(1)
		elseif GetIngredientSavedVar(index) == 0 then
			control:SetColor(0.7647,0.7647,0.7647,1)
			control:SetAlpha(1)
		end
	elseif option == 3 then
		if IsShiftKeyDown() == false then
			if GetIngredientSavedVar(index) == 1 then
				SetIngredientSavedVar(index, 0)
				CountIngredientTrackingList(0)
			elseif GetIngredientSavedVar(index) == 0 then
				SetIngredientSavedVar(index, 1)
				CountIngredientTrackingList(1)
			end
			ESOMRL.SV.cOpts.junkunmarkedingredients = 0
			ESOMRL.SV.cOpts.destroyunmarkedingredients = 0
			IngredientPanelSetup()
		elseif IsShiftKeyDown() == true then
			local ChatEditControl = CHAT_SYSTEM.textEntry.editControl
			if not ChatEditControl:HasFocus() then 
				StartChatInput() 
			end
			ChatEditControl:InsertText(ESOMRL.IngredientLinks[index].link)
		end
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Generate the popup list tooltips.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function ListTooltips(control, text, option)
	if option == 1 then
		if ESOMRL.SV.cOpts.lttshow == 1 then
			local RecipeItemTooltip = RecipeItemTooltipControl
			for i = 1, #searchTable do
				local row = searchTable[i].pos
				local name = searchTable[i].text
				if text == name then
					local itemlink

					if setMode == 1 then
						if ESOMRL.SV.cOpts.tooltipstyle == 0 then
							itemLink = ESOMRL.ProvisioningLinks[row].link
						elseif ESOMRL.SV.cOpts.tooltipstyle == 1 then
							itemLink = ESOMRL.ProvisioningLinks[row].item
						end
					else
						if ESOMRL.SV.cOpts.tooltipstyle == 0 then
							itemLink = ESOMRL.FurnitureLinks[row].link
						elseif ESOMRL.SV.cOpts.tooltipstyle == 1 then
							itemLink = ESOMRL.FurnitureLinks[row].item
						end
					end
					InitializeTooltip(RecipeItemTooltip, ESOMRL_MainFrame, TOPLEFT, 10, -8, TOPRIGHT)
					PopupTooltip.SetLink(RecipeItemTooltip, itemLink)
				end
			end
		end
		control:SetAlpha(.5)
	elseif option == 2 then
		local RecipeItemTooltip = RecipeItemTooltipControl
		ClearTooltip(RecipeItemTooltip)
		control:SetAlpha(1)
	end
end

local function SelectTooltipStyle(option)
	if option == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
		if ESOMRL.SV.cOpts.tooltipstyle == 0 then
			SetTooltipText(InformationTooltip, L.ESOMRL_TTFOOD)
		elseif ESOMRL.SV.cOpts.tooltipstyle == 1 then
			SetTooltipText(InformationTooltip, L.ESOMRL_TTRECIPE)
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if ESOMRL.SV.cOpts.tooltipstyle == 0 then
			ESOMRL.SV.cOpts.tooltipstyle = 1
			ClearTooltip(InformationTooltip)
			InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_TTRECIPE)
		elseif ESOMRL.SV.cOpts.tooltipstyle == 1 then
			ESOMRL.SV.cOpts.tooltipstyle = 0
			ClearTooltip(InformationTooltip)
			InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
			SetTooltipText(InformationTooltip, L.ESOMRL_TTFOOD)
		end
	end
end

local function StationButtonTooltip(control, option)
	if option == 1 then
		InitializeTooltip(InformationTooltip, control, BOTTOMLEFT, 0, -11, TOPRIGHT)
		SetTooltipText(InformationTooltip, "Master Recipe List")
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	end
end

local function QualityTooltips(tier, opt)
	local text = ''
	if tier == 1 then
		text = L.ESOMRL_RANK1
	elseif tier == 2 then
		text = L.ESOMRL_RANK2
	elseif tier == 3 then
		text = L.ESOMRL_RANK3
	elseif tier == 4 then
		text = L.ESOMRL_RANK4
	elseif tier == 5 then
		text = L.ESOMRL_RANK5
	elseif tier == 6 then
		text = L.ESOMRL_RANK6
	end

	if opt == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, text)
	elseif opt == 2 then
		ClearTooltip(InformationTooltip)
	end
end

local function TrackShownTooltips(opt)
	if opt == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, L.ESOMRL_TRACKSHOWN)
	elseif opt == 2 then
		ClearTooltip(InformationTooltip)
	end
end

local function SpecialRecipeTooltips(n1, opt)
	local text = ''
	if n1 == 1 then
		text = L.ESOMRL_PSIJIC
	elseif n1 == 2 then
		text = L.ESOMRL_ORSINIUM
	elseif n1 == 3 then
		text = L.ESOMRL_WITCHFEST
	elseif n1 == 4 then
		text = L.ESOMRL_NLFEST
	elseif n1 == 5 then
		text = L.ESOMRL_JESTFEST
	elseif n1 == 6 then
		text = L.ESOMRL_CLOCKWORK
	end

	if opt == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameListFrame, BOTTOMLEFT, 26, 28, BOTTOMRIGHT)
		SetTooltipText(InformationTooltip, text)
	elseif opt == 2 then
		ClearTooltip(InformationTooltip)
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Text search mouse events.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function TextBoxEvents(option)
	local control = ESOMRL_MainFrameListFrameSearchBox
	local controlBG = ESOMRL_MainFrameListFrameSearchBG
	if option == 1 then -- On Mouse Enter
		if editText == 0 or editText == 2 then
			controlBG:SetAlpha(0.5)
			if textInput == '' then
				control:SetText(L.ESOMRL_SEARCHBOX)
			else
				control:SetText(textInput)
			end
		end
	elseif option == 2 then -- On Mouse Exit
		if editText == 0 or editText == 2 then
			controlBG:SetAlpha(0)
			if control:GetText() == L.ESOMRL_SEARCHBOX then
				textInput = L.ESOMRL_SEARCHBOX
			end
			control:SetText('')
		end
	elseif option == 3 then -- On Focus Gained
		control:SetText('')
		textInput = ''
		editText = 1
		openTab = 50
		TextSearch()
	elseif option == 4 then -- On Focus Lost
		editText = 0
		textInput = control:GetText()
		control:SetText('')
		controlBG:SetAlpha(0)
		TextSearch(1)
	elseif option == 5 then -- On Text Changed
		if textInput ~= L.ESOMRL_SEARCHBOX and control:GetText() ~= L.ESOMRL_SEARCHBOX then
			TextSearch()
		end
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Handles populating the list of tracked ingredients based on those needed by currently tracked recipes.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function TrackedRecipeCheck()
	local ingredients = {}
	for r = 1, pRecipes do
		local savedvar = GetRecipeSavedVar(r, 1)
		if (savedvar == 1) or (savedvar == 2) then
			local linkcheck = ESOMRL.ProvisioningLinks[r].link
			local numIngredients = GetItemLinkRecipeNumIngredients(linkcheck)
			for i = 1, numIngredients do
				local ingredientName,_ = GetItemLinkRecipeIngredientInfo(linkcheck, i)
				ingredientName = LowerSpaceSpecial(ingredientName)
				if not Contains(ingredients, ingredientName) then
					table.insert(ingredients, #ingredients + 1, ingredientName)
				end
			end
		end
	end
	for a = 1, #ingredients do
		for b = 1, 50 do
			local namelookup = LowerSpaceSpecial(GetItemLinkName(ESOMRL.IngredientLinks[b].link))
			if namelookup == ingredients[a] then
				if GetIngredientSavedVar(b) == 0 then
					SetIngredientSavedVar(b, 1)
					CountIngredientTrackingList(1)
				end
			end
		end
	end
	IngredientPanelSetup()
end

local function TrackedRecipeIngredients(control, option)
	if option == 1 then
		if GetCVar('Language.2') == 'fr' then
			InitializeTooltip(InformationTooltip, control, TOPLEFT, 10, -65, BOTTOMRIGHT)
		elseif GetCVar('Language.2') == 'de' then
			InitializeTooltip(InformationTooltip, control, TOPLEFT, 10, -65, BOTTOMRIGHT)
		else
			InitializeTooltip(InformationTooltip, control, TOPLEFT, 10, -43, BOTTOMRIGHT)
		end
		SetTooltipText(InformationTooltip, L.ESOMRL_TRACKRING)
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		ESOMRL.SV.cOpts.junkunmarkedingredients = 0
		ESOMRL.SV.cOpts.destroyunmarkedingredients = 0
		SetTrackedIngredientCountText()
		RefreshViews()
		TrackedRecipeCheck()
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Setup button functions and handle the junking and deleting of unmarked items as per options.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function JunkUnmarkedRecipeFunction(option)
	if option == 1 then
		InitializeTooltip(InformationTooltip, ESOMRL_MainFrameCloseButton, TOPLEFT, 8, -44, BOTTOMRIGHT)
		if ESOMRL.SV.cOpts.destroyjunkrecipes == true then
			if ESOMRL.SV.cOpts.destroyunmarkedrecipes == 0 then
				SetTooltipText(InformationTooltip, L.ESOMRL_DESTROYURECIPE)
			elseif ESOMRL.SV.cOpts.destroyunmarkedrecipes == 1 then
				SetTooltipText(InformationTooltip, L.ESOMRL_DDESTROYURECIPE)
			end
		elseif ESOMRL.SV.cOpts.destroyjunkrecipes == false then
			if ESOMRL.SV.cOpts.junkunmarkedrecipes == 0 then
				SetTooltipText(InformationTooltip, L.ESOMRL_JUNKURECIPE)
			elseif ESOMRL.SV.cOpts.junkunmarkedrecipes == 1 then
				SetTooltipText(InformationTooltip, L.ESOMRL_DJUNKURECIPE)
			end
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if ESOMRL.SV.cOpts.destroyjunkrecipes == true then
			if ESOMRL.SV.cOpts.destroyunmarkedrecipes == 0 then
				ESOMRL.SV.cOpts.destroyunmarkedrecipes = 1
				ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton1:SetHidden(true)
				ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton2:SetHidden(false)
				ClearTooltip(InformationTooltip)
				InitializeTooltip(InformationTooltip, ESOMRL_MainFrameCloseButton, TOPLEFT, 8, -44, BOTTOMRIGHT)
				SetTooltipText(InformationTooltip, L.ESOMRL_DDESTROYURECIPE)
			elseif ESOMRL.SV.cOpts.destroyunmarkedrecipes == 1 then
				ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
				ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton1:SetHidden(false)
				ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton2:SetHidden(true)
				ClearTooltip(InformationTooltip)
				InitializeTooltip(InformationTooltip, ESOMRL_MainFrameCloseButton, TOPLEFT, 8, -44, BOTTOMRIGHT)
				SetTooltipText(InformationTooltip, L.ESOMRL_DESTROYURECIPE)
			end
		elseif ESOMRL.SV.cOpts.destroyjunkrecipes == false then
			if ESOMRL.SV.cOpts.junkunmarkedrecipes == 0 then
				ESOMRL.SV.cOpts.junkunmarkedrecipes = 1
				ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton1:SetHidden(true)
				ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton2:SetHidden(false)
				ClearTooltip(InformationTooltip)
				InitializeTooltip(InformationTooltip, ESOMRL_MainFrameCloseButton, TOPLEFT, 8, -44, BOTTOMRIGHT)
				SetTooltipText(InformationTooltip, L.ESOMRL_DJUNKURECIPE)
			elseif ESOMRL.SV.cOpts.junkunmarkedrecipes == 1 then
				ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
				ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton1:SetHidden(false)
				ESOMRL_MainFrameListFrameBatchTrackingTrashUncheckedButton2:SetHidden(true)
				ClearTooltip(InformationTooltip)
				InitializeTooltip(InformationTooltip, ESOMRL_MainFrameCloseButton, TOPLEFT, 8, -44, BOTTOMRIGHT)
				SetTooltipText(InformationTooltip, L.ESOMRL_JUNKURECIPE)
			end
		end
	end
end

local function JunkUnmarkedIngredientFunction(option)
	if option == 1 then
		if GetCVar('Language.2') == 'fr' then
			InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -87, BOTTOMRIGHT)
		else
			InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -65, BOTTOMRIGHT)
		end
		if ESOMRL.SV.cOpts.destroyjunkingredients == true then
			if ESOMRL.SV.cOpts.destroyunmarkedingredients == 0 then
				SetTooltipText(InformationTooltip, L.ESOMRL_DESTROYUING)
			elseif ESOMRL.SV.cOpts.destroyunmarkedingredients == 1 then
				SetTooltipText(InformationTooltip, L.ESOMRL_DDESTROYUING)
			end
		elseif ESOMRL.SV.cOpts.destroyjunkingredients == false then
			if ESOMRL.SV.cOpts.junkunmarkedingredients == 0 then
				SetTooltipText(InformationTooltip, L.ESOMRL_JUNKUING)
			elseif ESOMRL.SV.cOpts.junkunmarkedingredients == 1 then
				SetTooltipText(InformationTooltip, L.ESOMRL_DJUNKUING)
			end
		end
	elseif option == 2 then
		ClearTooltip(InformationTooltip)
	elseif option == 3 then
		if ESOMRL.SV.cOpts.destroyjunkingredients == true then
			if ESOMRL.SV.cOpts.destroyunmarkedingredients == 0 then
				ESOMRL.SV.cOpts.destroyunmarkedingredients = 1
				ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton1:SetHidden(true)
				ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton2:SetHidden(false)
				ClearTooltip(InformationTooltip)
				if GetCVar('Language.2') == 'fr' then
					InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -85, BOTTOMRIGHT)
				else
					InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -63, BOTTOMRIGHT)
				end
				SetTooltipText(InformationTooltip, L.ESOMRL_DDESTROYUING)
			elseif ESOMRL.SV.cOpts.destroyunmarkedingredients == 1 then
				ESOMRL.SV.cOpts.destroyunmarkedingredients = 0
				ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton1:SetHidden(false)
				ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton2:SetHidden(true)
				ClearTooltip(InformationTooltip)
				if GetCVar('Language.2') == 'fr' then
					InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -85, BOTTOMRIGHT)
				else
					InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -63, BOTTOMRIGHT)
				end
				SetTooltipText(InformationTooltip, L.ESOMRL_DESTROYUING)
			end
		elseif ESOMRL.SV.cOpts.destroyjunkingredients == false then
			if ESOMRL.SV.cOpts.junkunmarkedingredients == 0 then
				ESOMRL.SV.cOpts.junkunmarkedingredients = 1
				ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton1:SetHidden(true)
				ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton2:SetHidden(false)
				ClearTooltip(InformationTooltip)
				if GetCVar('Language.2') == 'fr' then
					InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -85, BOTTOMRIGHT)
				else
					InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -63, BOTTOMRIGHT)
				end
				SetTooltipText(InformationTooltip, L.ESOMRL_DJUNKUING)
			elseif ESOMRL.SV.cOpts.junkunmarkedingredients == 1 then
				ESOMRL.SV.cOpts.junkunmarkedingredients = 0
				ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton1:SetHidden(false)
				ESOMRL_MainFrameIngredientsFrameTrashUncheckedButton2:SetHidden(true)
				ClearTooltip(InformationTooltip)
				if GetCVar('Language.2') == 'fr' then
					InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -85, BOTTOMRIGHT)
				else
					InitializeTooltip(InformationTooltip, ESOMRL_MainFrameIngredientsFrameListTooltipButton, TOPLEFT, 10, -63, BOTTOMRIGHT)
				end
				SetTooltipText(InformationTooltip, L.ESOMRL_JUNKUING)
			end
		end
	end
end

local function OnInventorySlotUpdate(eventCode, bagId, slotId, isNewItem, itemSoundCategory, updateReason)
	if playerLoaded == 0 then return end
	if isNewItem ~= true then return end
	if bagId ~= BAG_BACKPACK then return end
	if updateReason ~= INVENTORY_UPDATE_REASON_DEFAULT then return end
	if IsUnderArrest() then return end
	if Roomba and Roomba.WorkInProgress and Roomba.WorkInProgress() then return end
	if IsItemJunk(bagId, slotId) then return end
	local icon, stackCount, sellPrice, meetsUsageRequirements, locked, equipType, itemStyle, quality = GetItemInfo(bagId, slotId)
	if stackCount < 1 then return end
	local itemLink = GetItemLink(bagId, slotId)
	local itemType = GetItemLinkItemType(itemLink)
	local itemId = select(4, ZO_LinkHandler_ParseLink(itemLink))
	if ESOMRL.SV.cOpts.ignorestolen and IsItemLinkStolen(itemLink) then
		if ESOMRL.SV.cOpts.debugmode == true then
			if itemType == ITEMTYPE_INGREDIENT then
				d('\'' .. stackCount .. 'x ' .. LanguageFormat(GetItemLinkName(itemLink)) .. '\' ' .. L.ESOMRL_ISTOLENM)
			elseif itemType == ITEMTYPE_RECIPE then
				d('\'' .. LanguageFormat(GetItemLinkName(itemLink)) .. '\' ' .. L.ESOMRL_ISTOLENM)
			end
		end
	return end
	if itemType == ITEMTYPE_INGREDIENT then
		local key = LookupPosition(ESOMRL.IngredientMaterials, tonumber(itemId))
		if key ~= 0 then
			if itemId == ESOMRL.IngredientLinks[key].id then
				if GetIngredientSavedVar(key) == 0 then
					if ESOMRL.SV.cOpts.destroyjunkingredients == true then
						if ESOMRL.SV.cOpts.destroyunmarkedingredients == 1 then
							if stackCount <= ESOMRL.SV.cOpts.maxjunkstack then
								DestroyItem(bagId, slotId)
								if ESOMRL.SV.cOpts.debugmode == true then
									d('\'' .. stackCount .. 'x ' .. LanguageFormat(GetItemLinkName(itemLink)) .. '\' ' .. L.ESOMRL_IDESTROYM)
								end
							end
						end
					elseif ESOMRL.SV.cOpts.destroyjunkingredients == false then
						if ESOMRL.SV.cOpts.junkunmarkedingredients == 1 then
							if not IsItemJunk(bagId, slotId) then
								if stackCount <= ESOMRL.SV.cOpts.maxjunkstack then	
									SetItemIsJunk(bagId, slotId, true)
									if ESOMRL.SV.cOpts.debugmode == true then
										d('\'' .. stackCount .. 'x ' .. LanguageFormat(GetItemLinkName(itemLink)) .. '\' ' .. L.ESOMRL_IJUNKM)
									end
								end
							end
						end
					end
				end
			end
		end
	elseif itemType == ITEMTYPE_RECIPE then
		local furniture = 0
		local key = LookupPosition(ESOMRL.RecipeMaterials, itemId)

		if key == 0 then
			local checkKey = LookupPosition(ESOMRL.RecipeFMaterials, itemId)
			if checkKey ~= 0 then
				key = checkKey
				furniture = 1
			else
				return
			end
		end

		local quality = GetItemLinkQuality(itemLink)
		local checktracking = (furniture == 0) and GetRecipeSavedVar(key, 1) or GetRecipeSavedVar(key, 100)
		local linkDB = (furniture == 0) and ESOMRL.ProvisioningLinks or ESOMRL.FurnitureLinks

		if itemId == linkDB[key].id then
			if checktracking == 0 or checktracking == 3 then
				if ESOMRL.SV.cOpts.destroyjunkrecipes == true then
					if ESOMRL.SV.cOpts.destroyunmarkedrecipes == 1 then
						if quality < ESOMRL.SV.cOpts.maxjunkquality then
							DestroyItem(bagId, slotId)
							if ESOMRL.SV.cOpts.debugmode == true then
								d('\'' .. LanguageFormat(GetItemLinkName(itemLink)) .. '\' ' .. L.ESOMRL_IDESTROYM)
							end
						end
					end
				elseif ESOMRL.SV.cOpts.destroyjunkrecipes == false then
					if ESOMRL.SV.cOpts.junkunmarkedrecipes == 1 then
						if not IsItemJunk(bagId, slotId) then
							if quality < ESOMRL.SV.cOpts.maxjunkquality then
								SetItemIsJunk(bagId, slotId, true)
								if ESOMRL.SV.cOpts.debugmode == true then
									d('\'' .. LanguageFormat(GetItemLinkName(itemLink)) .. '\' ' .. L.ESOMRL_IJUNKM)
								end
							end
						end
					end
				end
			end
		end
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Set up the Addon Settings options panel.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function CreateSettingsWindow(addonName)

	local panelData = {
		type					= 'panel',
		name					= 'ESO Master Recipe List',
		displayName				= '|cFEE854ESO|r |cFF9900Master Recipe List|r',
		author					= ESOMRL.Author,
		version					= ESOMRL.Version,
		registerForRefresh		= true,
		registerForDefaults		= true,
	}

	local optionsData = {
	{
		type			= 'submenu',
		name			= L.ESOMRL_CHAROPT,
		tooltip			= L.ESOMRL_CHAROPT,
		controls		= {
						[1] = {
							type			= 'checkbox',
							name			= L.ESOMRL_ETRACKING,
							tooltip			= L.ESOMRL_TRACKDESC,
							getFunc			= function() return ESOMRL.SV.cOpts.trackChar end,
							setFunc			= function(value)
												ESOMRL.SV.cOpts.trackChar = value
												InitKnown()
											end,
							warning			= L.ESOMRL_TRACKWARN,
							default			= cOpts.trackChar,
						},
						[2] = {
							type			= 'checkbox',
							name			= '|cff0000' .. L.ESOMRL_RDESTROY .. '|r',
							tooltip			= L.ESOMRL_RDESTDESC,
							getFunc			= function() return ESOMRL.SV.cOpts.destroyjunkrecipes end,
							setFunc			= function(value)
												ESOMRL.SV.cOpts.junkunmarkedrecipes = 0
												ESOMRL.SV.cOpts.destroyunmarkedrecipes = 0
												ESOMRL.SV.cOpts.destroyjunkrecipes = value
												RecipePanelSetup()
											end,
							warning			= L.ESOMRL_RDESTROYWARN,
							default			= cOpts.destroyjunkrecipes,
						},
						[3] = {
							type			= 'checkbox',
							name			= '|cff0000' .. L.ESOMRL_IDESTROY .. '|r',
							tooltip			= L.ESOMRL_IDESTDESC,
							getFunc			= function() return ESOMRL.SV.cOpts.destroyjunkingredients end,
							setFunc			= function(value)
												ESOMRL.SV.cOpts.junkunmarkedingredients = 0
												ESOMRL.SV.cOpts.destroyunmarkedingredients = 0
												ESOMRL.SV.cOpts.destroyjunkingredients = value
												IngredientPanelSetup()
											end,
							warning			= L.ESOMRL_IDESTROYWARN,
							default			= cOpts.destroyjunkingredients,
						},
						[4] = {
							type			= 'checkbox',
							name			= '|c00ff00' .. L.ESOMRL_ISTOLEN .. '|r',
							tooltip			= L.ESOMRL_ISTOLEND,
							getFunc			= function() return ESOMRL.SV.cOpts.ignorestolen end,
							setFunc			= function(value) ESOMRL.SV.cOpts.ignorestolen = value end,
							default			= cOpts.ignorestolen,
						},
						[5] = {
							type			= 'checkbox',
							name			= '|cffff00' .. L.ESOMRL_DEBUGMODE .. '|r',
							tooltip			= L.ESOMRL_DEBUGDESC,
							getFunc			= function() return ESOMRL.SV.cOpts.debugmode end,
							setFunc			= function(value) ESOMRL.SV.cOpts.debugmode = value end,
							default			= cOpts.debugmode,
						},
						[6] = {
							type			= 'dropdown',
							name			= L.ESOMRL_DQUALITY,
							tooltip			= L.ESOMRL_DQUALITYDESC,
							choices			= qualityCap,
							getFunc			= function() return qualityCap[ESOMRL.SV.cOpts.maxjunkquality - 1] end,
							setFunc			= function(selected)
												for k,v in ipairs(qualityCap) do
													if v == selected then
														ESOMRL.SV.cOpts.maxjunkquality = k + 1
														break
													end
												end
											end,
							default			= qualityCap[cOpts.maxjunkquality],
						},
						[7] = {
							type			= 'slider',
							name			= L.ESOMRL_STACKSIZE,
							tooltip			= L.ESOMRL_STACKDESC,
							min				= 1,
							max				= 100,
							step			= 1,
							getFunc			= function() return ESOMRL.SV.cOpts.maxjunkstack end,
							setFunc			= function(value) ESOMRL.SV.cOpts.maxjunkstack = value end,
							default			= cOpts.maxjunkstack,
						},
						[8] = {
							type			= 'checkbox',
							name			= L.ESOMRL_TCHART,
							tooltip			= L.ESOMRL_TCHARTD,
							getFunc			= function() return ESOMRL.SV.cOpts.tchart end,
							setFunc			= function(value)
												ESOMRL.SV.cOpts.tchart = value
												RefreshViews()
											end,
							default			= cOpts.tchart,
						},
						[9] = {
							type			= 'dropdown',
							name			= L.ESOMRL_TCHAR,
							tooltip			= L.ESOMRL_TCHARD,
							choices			= nameList,
							getFunc			= function() return nameList[ESOMRL.SV.cOpts.trackingchar] end,
							setFunc			= function(selected)
												for k,v in ipairs(nameList) do
													if v == selected then
														ESOMRL.SV.cOpts.trackingchar = k
														break
													end
												end
												RefreshViews()
											end,
							default			= cOpts.trackingchar,
							disabled		= function() return not ESOMRL.SV.cOpts.tchart end,
						},
						[10] = {
							type			= 'dropdown',
							name			= L.ESOMRL_FTCHAR,
							tooltip			= L.ESOMRL_FTCHARD,
							choices			= nameList,
							getFunc			= function() return nameList[ESOMRL.SV.cOpts.rtrackingchar] end,
							setFunc			= function(selected)
												for k,v in ipairs(nameList) do
													if v == selected then
														ESOMRL.SV.cOpts.rtrackingchar = k
														break
													end
												end
												RefreshViews()
											end,
							default			= cOpts.rtrackingchar,
							disabled		= function() return not ESOMRL.SV.cOpts.tchart end,
						},
						[11] = {
							type			= 'checkbox',
							name			= L.ESOMRL_ALTIU,
							tooltip			= L.ESOMRL_ALTIUD,
							getFunc			= function() return ESOMRL.SV.cOpts.altknown end,
							setFunc			= function(value)
												ESOMRL.SV.cOpts.altknown = value
												RefreshViews()
											end,
							default			= cOpts.altknown,
							disabled		= function() return not ESOMRL.SV.cOpts.tchart end,
						},
						[12] = {
							type			= 'checkbox',
							name			= L.ESOMRL_ALTIS,
							tooltip			= L.ESOMRL_ALTISD,
							getFunc			= function() return ESOMRL.SV.cOpts.altknownself end,
							setFunc			= function(value)
												ESOMRL.SV.cOpts.altknownself = value
												RefreshViews()
											end,
							default			= cOpts.altknownself,
							disabled		= function()
												if ESOMRL.SV.cOpts.altknown and ESOMRL.SV.cOpts.tchart then return false else return true end
											end,
						},
						},
	},
	{
		type			= 'submenu',
		name			= L.ESOMRL_GLOBALOPT,
		tooltip			= L.ESOMRL_GLOBALOPT,
		controls		= {
						[1] = {
							type = 'checkbox',
							name = L.ESOMRL_SHOWKNOWN,
							tooltip = L.ESOMRL_SHOWKNOWND,
							getFunc = function() return ESOMRL.ASV.aOpts.known end,
							setFunc = function(value) ESOMRL.ASV.aOpts.known = value end,
							width = 'full',
							default = aOpts.known,
						},
						[2] = {
							type = 'checkbox',
							name = L.ESOMRL_SHOWINGFOOD,
							tooltip = L.ESOMRL_SHOWINGFOODD,
							getFunc = function() return ESOMRL.ASV.aOpts.ingfood end,
							setFunc = function(value) ESOMRL.ASV.aOpts.ingfood = value end,
							width = 'full',
							default = aOpts.ingfood,
						},
						[3] = {
							type = 'checkbox',
							name = L.ESOMRL_NOFILTERS,
							tooltip = L.ESOMRL_NOFILTERSD,
							getFunc = function() return ESOMRL.ASV.aOpts.noFilters end,
							setFunc = function(value) ESOMRL.ASV.aOpts.noFilters = value end,
							width = 'full',
							default = aOpts.noFilters,
						},
						[4] = {
							type			= 'checkbox',
							name			= L.ESOMRL_INVENTORYI,
							tooltip			= L.ESOMRL_INVENTORYD,
							getFunc			= function() return ESOMRL.ASV.aOpts.inventoryicons end,
							setFunc			= function(value)
												ESOMRL.ASV.aOpts.inventoryicons = value
												RefreshViews()
											end,
							default			= aOpts.inventoryicons,
						},
						[5] = {
							type			= 'slider',
							name			= L.ESOMRL_ICONPOSI,
							tooltip			= L.ESOMRL_ICONPOSID,
							min				= -380,
							max				= 100,
							step			= 1,
							getFunc			= function() return ESOMRL.ASV.aOpts.bagiconoffset end,
							setFunc			= function(value)
												ESOMRL.ASV.aOpts.bagiconoffset = value
												RefreshViews()
											end,
							default			= aOpts.bagiconoffset,
							disabled		= function() return ESOMRL.SV.cOpts.fcoitemsaver end,
						},
						[6] =  {
							type			= 'slider',
							name			= L.ESOMRL_ICONPOSB,
							tooltip			= L.ESOMRL_ICONPOSBD,
							min				= -30,
							max				= 145,
							step			= 1,
							getFunc			= function() return ESOMRL.ASV.aOpts.gstoreiconoffset end,
							setFunc			= function(value)
												ESOMRL.ASV.aOpts.gstoreiconoffset = value
												RefreshViews()
											end,
							default			= aOpts.gstoreiconoffset,
						},
						[7] =  {
							type			= 'slider',
							name			= L.ESOMRL_ICONPOSL,
							tooltip			= L.ESOMRL_ICONPOSLD,
							min				= -30,
							max				= 145,
							step			= 1,
							getFunc			= function() return ESOMRL.ASV.aOpts.glistingiconoffset end,
							setFunc			= function(value)
												ESOMRL.ASV.aOpts.glistingiconoffset = value
												RefreshViews()
											end,
							default			= aOpts.glistingiconoffset,
						},
						[8] = {
							type			= 'checkbox',
							name			= L.ESOMRL_SSTATS,
							tooltip			= L.ESOMRL_SSTATSD,
							getFunc			= function() return ESOMRL.ASV.aOpts.stationstats end,
							setFunc			= function(value)
												ESOMRL.ASV.aOpts.stationstats = value
											end,
							default			= aOpts.stationstats,
						},
						[9] = {
							type			= 'iconpicker',
							name			= L.ESOMRL_SSTATICONS,
							tooltip			= L.ESOMRL_SSTATICONSD,
							choices			= DB.StatOptions,
							choicesTooltips	= DB.StatOptionTooltips,
							getFunc			= function() return DB.StatOptions[ESOMRL.ASV.aOpts.stationicons] end,
							setFunc			= function(texturePath)
												local textureId = GetTextureId(texturePath)
												if textureId ~= 0 then
													ESOMRL.ASV.aOpts.stationicons = textureId
													SwitchStationIcons(textureId)
												end
											end,
							maxColumns		= 2,
							visibleRows		= 1,
							iconSize		= 64,
							default			= DB.StatOptions[aOpts.stationicons],
							disabled		= function() return not ESOMRL.ASV.aOpts.stationstats end,
						},
						},
	},
	{
		type			= 'submenu',
		name			= L.ESOMRL_ADDONS,
		tooltip			= L.ESOMRL_ADDONSD,
		controls		= {
						[1] = {
							type			= 'checkbox',
							name			= L.ESOMRL_FCOITEM,
							tooltip			= L.ESOMRL_FCOITEMD,
							getFunc			= function() return ESOMRL.SV.cOpts.fcoitemsaver end,
							setFunc			= function(value)
												ESOMRL.SV.cOpts.fcoitemsaver = value
												RefreshViews()
											end,
							default			= cOpts.fcoitemsaver,
							disabled		= function() return not FCOItemSaverActive end,
						},
						}
	}
	}

	local LAM = LibStub('LibAddonMenu-2.0')
	LAM:RegisterAddonPanel('ESOMRL_Panel', panelData)
	LAM:RegisterOptionControls('ESOMRL_Panel', optionsData)
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Initialize the addon and the interactive recipe scroll list, and set up language strings and formatting.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function InitMain()
	local control = GetControl('ESOMRL_MainFrameListFrameList')
	ZO_ScrollList_AddDataType(control, 1 , 'ESOMRL_ListItemTemplate', 20,  SetListItem)
end

local function InitRecipeTooltip(control)
	if not control then return end
	RecipeItemTooltipControl = control
	control:SetParent(PopupTooltipTopLevel)
end

local function InitIngredientTooltip(control)
	if not control then return end
	IngredientItemTooltipControl = control
	control:SetParent(PopupTooltipTopLevel)
end

local function InitValues()
	qualityCap[1] = '|c00ff00' .. L.ESOMRL_QUALITY1 .. '|r'
	qualityCap[2] = '|c3a92ff' .. L.ESOMRL_QUALITY2 .. '|r'
	qualityCap[3] = '|ca02ef7' .. L.ESOMRL_QUALITY3 .. '|r'
end

local function ShowMain(option)
	if option == 'reposition' then
		ESOMRL.SV.cOpts.xpos = 0
		ESOMRL.SV.cOpts.ypos = 0
		RestorePosition()
	else
		ESOMRL.SetLanguageFormat()
		local control = GetControl('ESOMRL_MainFrame')
		if ( control:IsHidden() ) then
			SCENE_MANAGER:ToggleTopLevel(ESOMRL_MainFrame)
			ESOMRL_MainFrameSelectionFrame:SetHidden(false)
			ESOMRL_MainFrameFurnitureFrame:SetHidden(true)
			ESOMRL_MainFrameListFrame:SetHidden(false)
			ESOMRL_MainFrameIngredientsFrame:SetHidden(true)
			ESOMRL_MainFrameFoodButton:SetHidden(true)
			ESOMRL_MainFrameFurnitureButton:SetHidden(false)
			ESOMRL_MainFramePageButton:SetHidden(false)
			ESOMRL_MainFrameListFrameQuality:SetHidden(false)
			ESOMRL_MainFramePageButton:ClearAnchors()
			ESOMRL_MainFramePageButton:SetAnchor(TOPLEFT, ESOMRL_MainFrameFurnitureButton, TOPRIGHT, 0, 0)
			setPage = 1
			setMode = 1
			openTab = 1
			RestorePosition()
			NavigateScrollList(1)
		else
			SCENE_MANAGER:ToggleTopLevel(ESOMRL_MainFrame)
		end
		SetTrackedRecipeCountText()
		RecipePanelRestore()
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Global function to check if MRL is tracking a recipe or ingredient.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
function ESOMRL.ISMRLTracking(itemLink)
	if itemLink ~= nil then
		local itemIdT = {select(4, ZO_LinkHandler_ParseLink(itemLink))}
		local itemId = tonumber(itemIdT[1])
		if ESOMRL.IngredientMaterials[itemId] ~= nil then
			local key = ESOMRL.IngredientMaterials[itemId].position
			if GetIngredientSavedVar(key) == 1 then
				return true
			end
		elseif ESOMRL.RecipeMaterials[itemId] ~= nil then
			local key = ESOMRL.RecipeMaterials[itemId].position
			local checktracking = GetRecipeSavedVar(key, 1)
			if checktracking == 1 or checktracking == 2 then return true end
		elseif ESOMRL.RecipeFMaterials[itemId] ~= nil then
			local key = ESOMRL.RecipeFMaterials[itemId].position
			local checktracking = GetRecipeSavedVar(key, 100)
			if checktracking == 1 or checktracking == 2 then return true end
		end
	end
	return false
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Handle function calls from XML.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
function ESOMRL.XMLNavigation(option, control, text, n1, n2)
-- List click and tooltips.
	if option == 001 then
		ListTooltips(control, text, n1)
	elseif option == 002 then
		RecipeListClick(text, n1)
-- Main Frame functions.
	elseif option == 101 then
		OnMoveStop()
	elseif option == 102 then
		InitMain()
	elseif option == 103 then
		InitRecipeTooltip(control)
	elseif option == 104 then
		InitIngredientTooltip(control)
	elseif option == 105 then
		CloseMain(control, n1)
	elseif option == 106 then
		ToggleTooltips(control, n1)
	elseif option == 107 then
		InfoTooltip(n1)
	elseif option == 108 then
		SwitchMode(control, n1)
	elseif option == 109 then
		SwitchMode(control, n1)
	elseif option == 110 then
		SwitchPage(control, n1)
-- Ingredient Frame functions.
	elseif option == 201 then
		IngredientListClick(control, n1, n2)
	elseif option == 202 then
		ClearTrackedIngredients()
	elseif option == 203 then
		TrackedRecipeIngredients(control, n1)
	elseif option == 204 then
		SelectAllIngredients(control, n1)
	elseif option == 205 then
		JunkUnmarkedIngredientFunction(n1)
	elseif option == 206 then
		FindIngredientRecipes(n1)
-- Left side navigation.
	elseif option == 301 then
		ClearTrackedRecipes()
	elseif option == 302 then
		WritButton(control, n1)
	elseif option == 303 then
		GetWrits()
	elseif option == 304 then
		if IsShiftKeyDown() == true then
			TrackNavigationTier(n1)
		end
		NavigateScrollList(n1)
	elseif option == 305 then
		SelectTooltipStyle(n1)
	elseif option == 306 then
		RecipeOptionPanel(n1)
	elseif option == 307 then
		SpecialRecipeTooltips(n1, n2)
-- Improvement quality navigation
	elseif option == 308 then
		QualityTooltips(n1, n2)
	elseif option == 309 then
		if IsShiftKeyDown() == true then
			TrackQuality(n1)
		end
		NavigateQuality(n1)
-- Track all listed recipes
	elseif option == 310 then
		TrackShownTooltips(n1)
	elseif option == 311 then
		TrackShown()
-- Text search options
	elseif option == 401 then
		TextBoxEvents(n1)
-- Open Main Window
	elseif option == 501 then
		ShowMain()
-- Recipe config menu functions.
	elseif option == 601 then
		TrackUnknownRecipes(n1)
	elseif option == 602 then
		TrackAllRecipes(n1)
	elseif option == 603 then
		JunkUnmarkedRecipeFunction(n1)
	elseif option == 604 then
		RemoveCharacter(n1)
-- Cooking station controls.
	elseif option == 701 then
		StationButtonTooltip(control, n1)
	elseif option == 702 then
		HighlightTracked(control, n1)
	elseif option == 703 then
		ToggleStationTooltips(control, n1)
	elseif option == 704 then
		StationSearch(control, n1)
	elseif option == 705 then
		ResetStationNavigation(control, n1)
	end
end

-------------------------------------------------------------------------------------------------------------------------------------------------------------------
-- Addon initialization and database maintenance.
-------------------------------------------------------------------------------------------------------------------------------------------------------------------
local function ResetVariables() -- When recipe database changes, to avoid bloating file with versions.
	for k,v in pairs(dataNames) do
		if v ~= GetUnitName("player") then
			table.remove(nameList, FindName(nameList, v))
		end
		for p = 1, pRecipes do
			if Contains(ESOMRL.ASV.pRecipeKnown[p].names, v) then
				table.remove(ESOMRL.ASV.pRecipeKnown[p].names, FindName(ESOMRL.ASV.pRecipeKnown[p].names, v))
			end
		end
		for f = 1, fRecipes do
			if Contains(ESOMRL.ASV.fRecipeKnown[f].names, v) then
				table.remove(ESOMRL.ASV.fRecipeKnown[f].names, FindName(ESOMRL.ASV.fRecipeKnown[f].names, v))
			end
		end
	end
	currentChar = GetUnitName("player") -- Set the addon-global active name to the current character.
	charnameDropdown.dropdown:ClearItems() -- Clear the list of names in the dropdown menu so we can rebuild.
	for k,v in pairs(nameList) do -- Rebuild using the names table we just removed a character from.
		local entry = charnameDropdown.dropdown:CreateItemEntry(v, OnCharacterSelected)
		charnameDropdown.dropdown:AddItem(entry)
	end
	charnameDropdown.dropdown:SetSelectedItem(currentChar) -- Set the dropdown selected item to the current character.
end

local function DBMaintenance() -- Checks if update to the database tables is necessary.
	if ESOMRL.ASV.aOpts.version < 1.551 then
		ResetVariables()
		d("****************************************************")
		d(L.ESOMRL_DBRESET)
		d("****************************************************")
		ESOMRL.ASV.aOpts.version = 1.551
	end
	InitKnown()
end

local function OnAddonLoaded(event, addonName)
	--Init, bindings and saved variables
	if addonName ~= ESOMRL.Name then return end
	EVENT_MANAGER:UnregisterForEvent(ESOMRL.Name, EVENT_ADD_ON_LOADED)
	SCENE_MANAGER:RegisterTopLevel(ESOMRL_MainFrame, false)
	ZO_CreateStringId('SI_BINDING_NAME_TOGGLE_RECIPE_WINDOW', 'Toggle Recipe Window')
	cOpts, aOpts = ESOMRL.SetupVars(pRecipes, fRecipes)

	--Additional init values
	FCOItemSaverActive = (FCOItemSaver_Settings ~= nil)
	if not FCOItemSaverActive then ESOMRL.SV.cOpts.fcoitemsaver = false end

	--Run startup functions
	SetupCharacterDropbox()
	InitValues()
	HookBags()
	HookTooltips()
	HookStation()
	DBMaintenance()
	StationWritCheck()
	CreateSettingsWindow(addonName)
end
local function Init(eventCode) playerLoaded = 1 end

SLASH_COMMANDS['/recipes'] = ShowMain
SLASH_COMMANDS['/mrl'] = function(option) ShowMain(option) end
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_PLAYER_ACTIVATED, Init)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_ADD_ON_LOADED, OnAddonLoaded)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_RECIPE_LEARNED, InitKnown)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_INVENTORY_SINGLE_SLOT_UPDATE, OnInventorySlotUpdate)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_TRADING_HOUSE_RESPONSE_RECEIVED, HookTradeHouse)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_OPEN_STORE, HookBuyback)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_QUEST_ADDED, OnQuestAdded)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_QUEST_REMOVED, OnQuestRemoved)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_QUEST_COMPLETE, OnQuestComplete)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_QUEST_CONDITION_COUNTER_CHANGED, OnQuestConditionCounterChanged)
EVENT_MANAGER:RegisterForEvent(ESOMRL.Name, EVENT_CRAFTING_STATION_INTERACT, OnCraftingStationInteract)
